function [jac] = IpoptFD_0(x,Params,lam,FuncName,FuncName2,addParams,SparseStructure,method) 
//**********************************************************************
//* $Id$ 
//* $Source$
//*
//* Part of SCILAB Mex interface for IPOPT
//*
//* This code is published under the Common Public License
//* http://www.opensource.org/licenses/cpl.php
//*
//* Copyright (C) 2004; All Rights Reserved
//*
//* Claas Michalik <Michalik<at>LPT<dot>RWTH-Aachen<dot>de>
//* Process Systems Engineering Institute, RWTH Aachen
//*
//* Steinar Hauan <hauan@cmu.edu>
//* Dept of ChemE, Carnegie Mellon University
//* Modifications to Scilab Interface made by
//* Edson Cordeiro do Valle <edsoncv@vrtech.com.br>
//* Departamento de Desenvolvimento e Tecnologia, VRTech Tecnologias Industriais LTDA.
//* www.vrtech.com.br
//**********************************************************************



h1_start      = addParams(1);
h2_start      = addParams(2);
eps1          = addParams(3);
eps2          = addParams(4);
maxiter1      = addParams(5);
maxiter2      = addParams(6);
printWarnings = addParams(7);

// x = x(:);
// lam = lam(:);

// Input variables:
//  x          : point of interest            
//  lam        : values of multipliers      
//  h1_start   : initial perturbation size    [default = 0]
//               for first order derivatives
//               if = 0 the perturbation size
//               is chosen automatically
//  h1_start   : initial perturbation size    [default = 0]
//               for first order derivatives
//               if = 0 the perturbation size
//               is chosen automatically       
//  eps1       : desired accuracy of result   [default = 1E-9]
//               for first order derivatives
//  eps2       : desired accuracy of result   [default = 1E-9]
//               for second order derivatives
//  maxiter1   : max number of iterations for [default = 1] 
//               first order derivatives     
//  maxiter2   : max number of iterations for [default = 1]
//               second order derivatives
// printWarnings 0 --> do not print warnings  [default = 0]
//               1 --> print warnings   
//  method     : 10 = Calculate Gradient of the Objective using
//               finite differences, simple algorithm 
//               11 = Calculate Jacobian of the Objective using
//               finite differences, complex algorithm 
//               20 = Calculate the full Jacobian of the constraints
//               using finite differences, simple algorithm
//               21 = Calculate the full Jacobian of the constraints
//               using finite differences, complex algorithm 
//               22 = Calculate the sparse Jacobian of the
//               constraints using finite differences, simple
//               algorithm 
//               23 = Calculate the sparse Jacobian of the
//               constraints using finite differences, complex
//               algorithm
//               30 = Calculate the full Hessian of the Lagrangian
//               using finite differences
//               31 = Calculate the sparse Hessian of the Lagrangian
//               using finite differences 
// 
//  FuncName  :  for 10 <= method <= 23 : name of function to
//                                       derive
//               else                   : name of Objective function 
//  FuncName2 :  for 10 <= method <= 23 : meaningless
//               else                   : name of constraints
//                                       function 
// addParams   : additional parmaters  


// check user-supplied function name

x_size = length(x);
y_size = length(evstr(FuncName+"(x,Params)"));

if ( maxiter1 <= 0 )
  maxiter1 = 1;
end
if ( maxiter2 <= 0 )
  maxiter2 = 1;
end

if ( length(SparseStructure) == 1 )
  IpoptFDJacobianSparsityPattern = ones(y_size,x_size);
  IpoptFDHessianSparsityPattern  = tril(ones(x_size,x_size));
else
  IpoptFDHessianSparsityPattern  = SparseStructure;
  IpoptFDJacobianSparsityPattern = SparseStructure;
end


chk = exists(FuncName);
if ( chk <= 0 )
//  fprintf([1,'Unable to open function  %s'],FuncName);
elseif ( chk == 1 )
//  fprintf([1,'The given function %s is a VARIABLE'],FuncName);
end

// preallocate output matrix
x_size = length(x);
y_size = length(evstr(FuncName+"(x,Params)"));
if ( method == 10 | method == 11 )
  jac = zeros(1,x_size);
elseif ( 20 <= method & method <= 23)
  jac = zeros(y_size,x_size);
else
  jac = (zeros(x_size,x_size));
end

// ********************************************************
// calculations for 1st order derivatives using full 
// matrices and the simple (less accurate) fd approach
// ********************************************************
if (method == 10 | method == 20 )
  xplus  = x;
  xminus = x;
  //value  = evstr(FuncName+"(x,Params)"));
  nmode   = 1;
  
  for j = 1:y_size
    for i = 1:x_size
      h1_realstart = h1_start;
      if ( h1_start <= 0 )
	if ( maxiter1 <= 1 )
	  h1_realstart = (%eps^(1/3))*max( 0.2,abs(x(i)) );
        else
	  h1_realstart = 2*(%eps^(1/3))*max(0.2,abs(x(i)) );
        end
      end
      h      = h1_realstart;
      acc    = 1E9;
      accold = 1e10;
      count  = 0;
      while ( abs(acc) > eps1 & count < maxiter1 )
	temp       = x(i) + h;
	h          = temp - x(i);
	xplus(i)   = x(i) + h;
	xminus(i)  = x(i) - h;

//	valueplus  = feval(FuncName,xplus);
	valueplus  = evstr(FuncName+"(xplus,Params)");
//	valueminus = feval(FuncName,xminus);
	valueminus = evstr(FuncName+"(xminus,Params)");
	jac(j,i)   = (valueplus(j)-valueminus(j))/(2*h);
	if ( maxiter1 > 1 & count > 0)
	  if ( nmode > 0 )
	    h = h/2;
	  else
	    h = h*7;
	  end
	  acc = abs(jac(j,i)-oldvalue);
	  if ( accold < acc )
	    if ( nmode > 0 )
	      h = h*2;
	    else
	      h = h/9;
	    end
	    nmode = -nmode;
	  end
	end
	accold = acc;
	oldvalue = jac(j,i);
	count = count+1;
      end
      if ( maxiter1 > 1 & acc > eps1 & printWarnings > 0 )
	fprintf([1,'WARNING: Desired accuracy for first' ...
		 ' derivatives (%e) could not be  obtained' ...
		 ' within max number of iterations (%e).' ...
		 ' Final error  estimate is: %e\n'],eps2, ...
		maxIter1,acc)
      end
      xplus  = x;
      xminus = x;
    end
  end
  
  // ********************************************************
  // Calculations for first order derivatives using full
  // matrices and the more accurate fd approach
  // ********************************************************
elseif ( method == 11 | method == 21 )
  xplus   = x;
  x2plus  = x;
  x4plus  = x;
  xminus  = x;
  x2minus = x;
  x4minus = x;
  nmode    = 1;
  for j = 1:y_size
    for i = 1:x_size
      h1_realstart = h1_start;
      if ( h1_start <= 0 )
	if ( maxiter1 <= 1 )
	  h1_realstart = 0.25*( %eps^(1/7) )*( abs(x(i)) + 0.5 );
        else
          h1_realstart = 0.5*( %eps^(1/7) )*( abs(x(i)) + 0.5 );
        end
      end
      h      = h1_realstart;
      acc    = 1E9;
      accold = 1E10;
      count  = 0;
      while ( abs(acc) > eps1 & count < maxiter1 )
	temp  = x(i) + h;
	h     = temp - x(i);
	xplus(i)    = x(i) +   h;
	x2plus(i)   = x(i) + 2*h;
	x4plus(i)   = x(i) + 4*h;
	xminus(i)   = x(i) -   h;
	x2minus(i)  = x(i) - 2*h;
	x4minus(i)  = x(i) - 4*h;

//	valueplus   = feval(FuncName,xplus  );
	valueplus   = evstr(FuncName+"(xplus,Params)");

//	value2plus  = feval(FuncName,x2plus );
	value2plus   = evstr(FuncName+"(x2plus,Params)");
//	value4plus  = feval(FuncName,x4plus );

	value4plus   = evstr(FuncName+"(x4plus,Params)");
//	valueminus  = feval(FuncName,xminus );
	valueminus   = evstr(FuncName+"(xminus,Params)");
//	value2minus = feval(FuncName,x2minus);
	value2minus   = evstr(FuncName+"(x2minus,Params)");
//	value4minus = feval(FuncName,x4minus);
	value4minus   = evstr(FuncName+"(x4minus,Params)");
	f1 = (  valueplus(j) -  valueminus(j) )/(2*h);
	f2 = ( value2plus(j) - value2minus(j) )/(4*h);
	f4 = ( value4plus(j) - value4minus(j) )/(8*h);
	jac(j,i) = f1 + 0.4*(f1-f2) + (1/45)*(f1- 2*f2 + f4);
	if ( maxiter1 > 1 & count > 0 )
	  if ( nmode > 0 )
	    h = h/2;
	  else
	    h = h*7;
	  end
	  acc = abs(jac(j,i)-oldvalue);
	  if ( accold < acc )
	    if ( nmode > 0 )
	      h = h*2;
	    else
	      h = h/9;
	    end
	    nmode = -nmode;
	  end
	end
	accold   = acc;
	oldvalue = jac(j,i);
	count    = count+1;
      end
      if ( maxiter1 > 1 & acc > eps1 & printWarnings > 0 )
	fprintf([1,'WARNING: Desired accuracy for first' ...
		 ' derivatives (%e) could not be  obtained' ...
		 ' within max number of iterations (%e).' ...
		 ' Final error  estimate is: %e\n'],eps2, ...
		maxIter1,acc)      
      end
      xplus   = x;
      x2plus  = x;
      x4plus  = x;
      xminus  = x;
      x2minus = x;
      x4minus = x;
    end
  end
  
  // ****************************************************
  // Calculate sparse Jacobian using the simple approach
  // (faster, but less accurate)
  // ****************************************************
elseif ( method == 22 )
  xplus = x;
  xminus = x;
  //value = evstr(FuncName+"(x,Params)"));
  nmode = 1;
  for j = 1:y_size
    for i = 1:x_size
      if ( IpoptFDJacobianSparsityPattern(j,i) > 0 )
	h1_realstart = h1_start;
	if ( h1_start <= 0 )
	  if ( maxiter1 <= 1 )
	    h1_realstart = (%eps^(1/3))*max( 0.2,abs(x(i)) );
          else
	    h1_realstart = 2*(%eps^(1/3))*max(0.2,abs(x(i)) );
          end
	end
	h      = h1_realstart;
	accold = 1E10;
	acc    = 1E9;
	count  = 0;
	while ( abs(acc) > eps1 & count < maxiter1 )
	  temp       = x(i) + h;
	  h          = temp - x(i);
	  xplus(i)   = x(i) + h;
	  xminus(i)  = x(i) - h;
//	  valueplus  = feval(FuncName,xplus);
	  valueplus  = evstr(FuncName+"(xplus,Params)")
//	  valueminus = feval(FuncName,xminus);
	  valueminus  = evstr(FuncName+"(xminus,Params)")

	  jac(j,i)   = (valueplus(j)-valueminus(j))/(2*h);
	  if ( maxiter1 > 1 & count > 0 )
	    if ( nmode > 0 )
	      h = h/2;
	    else
	      h = h*7;
	    end
	    acc = abs(jac(j,i)-oldvalue);
	    if ( accold < acc )
	      if ( nmode > 0 )
		h = h*2;
	      else
		h = h/9;
	      end
	      nmode = -nmode;
	    end
	  end
	  accold = acc;
	  oldvalue = jac(j,i);
	  count = count+1;
	end
	if ( maxiter1 > 1 & acc > eps1 & printWarnings > 0 )
	  fprintf([1,'WARNING: Desired accuracy for first' ...
		   ' derivatives (%e) could not be  obtained' ...
		   ' within max number of iterations (%e).' ...
		   ' Final error  estimate is: %e\n'],eps2, ...
		  maxIter1,acc)
	end
	xplus  = x;
	xminus = x;
      end
    end
  end
  jac = sparse(jac);
  jac = mtlb_sparse(jac);
  
  // ****************************************************
  // Calculate sparse Jacobian using more accurate fd
  // method
  // ****************************************************
  elseif ( method == 23 )
    xplus   = x;
    x2plus  = x;
    x4plus  = x;
    xminus  = x;
    x2minus = x;
    x4minus = x;
    nmode    = 1;
    for j = 1:y_size
      for i = 1:x_size
	if ( IpoptFDJacobianSparsityPattern(j,i) > 0 )
	  h1_realstart = h1_start;
          if ( h1_start <= 0 )
	    if ( maxiter1 <= 1 )
	      h1_realstart = 0.25*( %eps^(1/7) )*( abs(x(i)) + 0.5 );
            else
              h1_realstart = 0.5*( %eps^(1/7) )*( abs(x(i)) + 0.5 );
            end
	  end
	  h      = h1_realstart;
	  accold = 1E10;
	  acc    = 1E9;
	  count  = 0;
	  while ( abs(acc) > eps1 & count < maxiter1 )
	    temp  = x(i) + h;
	    h     = temp - x(i);
	    xplus(i)    = x(i) +   h;
	    x2plus(i)   = x(i) + 2*h;
	    x4plus(i)   = x(i) + 4*h;
	    xminus(i)   = x(i) -   h;
	    x2minus(i)  = x(i) - 2*h;
	    x4minus(i)  = x(i) - 4*h;
//	    valueplus   = feval(FuncName,xplus  );
	    valueplus   = evstr(FuncName+"(xplus,Params)");

//	    value2plus  = feval(FuncName,x2plus );
	    value2plus   = evstr(FuncName+"(x2plus,Params)");
//	    value4plus  = feval(FuncName,x4plus );

	    value4plus   = evstr(FuncName+"(x4plus,Params)");
//	    valueminus  = feval(FuncName,xminus );
	    valueminus   = evstr(FuncName+"(xminus,Params)");
//	    value2minus = feval(FuncName,x2minus);
	    value2minus   = evstr(FuncName+"(x2minus,Params)");
//	    value4minus = feval(FuncName,x4minus);
	    value4minus   = evstr(FuncName+"(x4minus,Params)");
	   
	    f1 = (  valueplus(j) -  valueminus(j) )/(2*h);
	    f2 = ( value2plus(j) - value2minus(j) )/(4*h);
	    f4 = ( value4plus(j) - value4minus(j) )/(8*h);
	    jac(j,i) = f1 + 0.4*(f1-f2) + (1/45)*(f1- 2*f2 + f4);
	    if ( maxiter1 > 1 & count > 0 )
	      if ( nmode > 0 )
		h = h/2;
	      else
		h = h*7;
	      end
	      acc = abs(jac(j,i)-oldvalue);
	      if ( accold < acc )
		if ( nmode > 0 )
		  h = h*2;
		else
		  h = h/9;
		end
		nmode = -nmode;
	      end
	    end
	    accold   = acc;
	    oldvalue = jac(j,i);
	    count    = count+1;
	  end
	  if ( maxiter1 > 1 & acc > eps1 & printWarnings > 0 )
	    fprintf([1,'WARNING: Desired accuracy for first' ...
		     ' derivatives (%e) could not be  obtained' ...
		     ' within max number of iterations (%e).' ...
		     ' Final error  estimate is: %e\n'],eps2, ...
		    maxIter1,acc)      
	  end
	  xplus   = x;
	  x2plus  = x;
	  x4plus  = x;
	  xminus  = x;
	  x2minus = x;
	  x4minus = x;
	end
      end
    end
    jac = sparse(jac);  
    jac = mtlb_sparse(jac);

  // ****************************************************
  // Calcualte the full Hessian
  // ****************************************************
elseif ( method == 30 )
  if ( FuncName2 ~= '' )
    test = 0;
    chk = exists(FuncName2);
    if ( chk <= 0 )
//      fprintf([1,'Unable to open function %s'],FuncName2);
    elseif ( chk == 1 )
//      fprintf([1,'The given function %s is a VARIABLE'], FuncName2);
    end
  elseif ( FuncName2 == '' )
    test = 1;
    lam = zeros(length(lam),1);
  end
  //   xnew = x;
  for j = 1:x_size
    h_j = h2_start;
    if ( h2_start <= 0 )
      if ( maxiter2 <= 1 )
        h_j = (%eps^0.25)*max( 0.2,min( abs(x(j)),1E5 ) );
      else
        h_j = 2*(%eps^0.25)*max( 0.2,min( abs(x(j)),1E5 ) ); 
      end
    end
    temp = 2 + h_j;
    h_j = temp - 2;
    for i = 1:j
      accold = 1e10;
      acc    = 1e9;
      count  = 0;
      nmode   = 1;
      h_i    = h2_start;
      if ( h2_start <= 0 )
	if ( maxiter2 <= 1 )
	  h_i = (%eps^0.25)*max( 0.2,min( abs(x(i)),1E5 ) );
        else
	  h_i = 2*(%eps^0.25)*max( 0.2,min( abs(x(i)),1E5 ) ); 
        end
      end
      temp = 2 + h_i;
      h_i = temp - 2;
      while ( abs(acc) > eps2 & count < maxiter2 )
	xnew = x;
	xnew(i) = xnew(i) +   h_i;
	xnew(j) = xnew(j) +   h_j;
	xpp = xnew;
	xnew(j) = xnew(j) - 2*h_j;
	xpm = xnew;
	xnew(i) = xnew(i) - 2*h_i;
	xmm = xnew;
	xnew(j) = xnew(j) + 2*h_j;
	xmp = xnew;
	if ( test==0 )
//          c_xpp = feval(FuncName2,xpp);
            c_xpp = evstr(FuncName2+"(xpp,Params)");
//          c_xpm = feval(FuncName2,xpm);
            c_xpm = evstr(FuncName2+"(xpm,Params)");
//          c_xmp = feval(FuncName2,xmp);
            c_xmp = evstr(FuncName2+"(xmp,Params)")
//          c_xmm = feval(FuncName2,xmm);
            c_xmm = evstr(FuncName2+"(xmm,Params)");
//	  valuepp = feval(FuncName,xpp) + lam(:)'*c_xpp(:);
          valuepp = evstr(FuncName+"(xpp,Params)")+ lam(:)'*c_xpp(:); 
//	  valuepm = feval(FuncName,xpm) + lam(:)'*c_xpm(:);
          valuepm = evstr(FuncName+"(xpm,Params)")+ lam(:)'*c_xpm(:);
//	  valuemp = feval(FuncName,xmp) + lam(:)'*c_xmp(:);
          valuemp = evstr(FuncName+"(xmp,Params)")+ lam(:)'*c_xmp(:);
//	  valuemm = feval(FuncName,xmm) + lam(:)'*c_xmm(:);
          valuemm = evstr(FuncName+"(xmm,Params)")+ lam(:)'*c_xmm(:);

	else
//	  valuepp = feval(FuncName,xpp);
          valuepp = evstr(FuncName+"(xpp,Params)"); 
//	  valuepm = feval(FuncName,xpm);
          valuepm = evstr(FuncName+"(xpm,Params)"); 
//	  valuemp = feval(FuncName,xmp);
          valuemp = evstr(FuncName+"(xmp,Params)"); 

//	  valuemm = feval(FuncName,xmm);
          valuemm = evstr(FuncName+"(xmm,Params)"); 

	end
	jac(j,i) = (valuepp-valuepm-valuemp+valuemm)/(4*h_i*h_j);
	if ( maxiter2 > 1 )
	  if ( nmode > 0 )
	    h_i = h_i/2;
	    h_j = h_j/2;
	  else
	    h_i = h_i*7;
	    h_j = h_j*7;
	  end
	  if ( count > 0 )
	    acc = abs( jac(j,i)-oldvalue );
	    if ( accold < acc)
	      if ( nmode > 0 )
		h_i = h_i*2;
		h_j = h_j*2;
	      else 
		h_i = h_i/9;
		h_j = h_j/9;
	      end
	      nmode = -nmode;
	    end
	  end
	  accold = acc;
	  oldvalue = jac(j,i);
	end
	count = count+1;
      end
      if ( maxiter2 > 1 & acc > eps2 & printWarnings > 0 )
	fprintf([1,'WARNING: Desired accuracy for second' ...
		 ' derivatives (%e) could not be  obtained' ...
		 ' within max number of iterations (%e).' ...
		 ' Final error  estimate is: %e\n'],eps2, ...
		maxiter2,acc)
      end
    end
  end
   
  // ************************************************
  // Calculate Sparse Hessian
  // ************************************************
elseif ( method == 31 )
  if ( FuncName2 ~= '' )
    test = 0;
    chk = exists(FuncName2);
    if ( chk <= 0 )
 //     fprintf([1,'Unable to open function %s'], FuncName2);
    elseif ( chk == 1 )
 //     fprintf([1,'The given function %s is a VARIABLE'],FuncName2);
    end
  elseif ( FuncName2 == '' )
    test = 1;
    lam = zeros(length(lam),1);
  end
  //  xnew = x;
  for j = 1:x_size
    h_j = h2_start;
    if ( h2_start <= 0 )
      if ( maxiter2 <= 1 )
        h_j = (%eps^0.25)*max( 0.2,min( abs(x(j)),1E5 ) );
      else
        h_j = 2*(%eps^0.25)*max( 0.2,min( abs(x(j)),1E5 ) ); 
      end
    end
    temp = 2 + h_j;
    h_j = temp - 2;
    for i = 1:j
      if ( IpoptFDHessianSparsityPattern(j,i) > 0 ) 
	accold = 1e10;
	acc = 1e9;
	count = 0;
	h_i = h2_start;
	if ( h2_start <= 0 )
	  if ( maxiter2 <= 1 )
	    h_i = (%eps^0.25)*max( 0.2,min( abs(x(i)),1E5 ) );
          else
	    h_i = 2*(%eps^0.25)*max( 0.2,min( abs(x(i)),1E5 ) ); 
          end
	end
	temp = 2 + h_i;
	h_i = temp - 2;
	nmode = 1;
	while ( abs(acc) > eps2 & count < maxiter2 )
	  xnew = x;
	  xnew(i) = xnew(i) + h_i;
	  xnew(j) = xnew(j) + h_j;
	  xpp = xnew;
	  xnew(j) = xnew(j) - 2*h_j;
	  xpm = xnew;
	  xnew(i) = xnew(i) - 2*h_i;
	  xmm = xnew;
	  xnew(j) = xnew(j) + 2*h_j;
	  xmp = xnew;
	  if ( test == 0 )

//          c_xpp = feval(FuncName2,xpp);
            c_xpp = evstr(FuncName2+"(xpp,Params)");
//          c_xpm = feval(FuncName2,xpm);
            c_xpm = evstr(FuncName2+"(xpm,Params)");
//          c_xmp = feval(FuncName2,xmp);
            c_xmp = evstr(FuncName2+"(xmp,Params)")
//          c_xmm = feval(FuncName2,xmm);
            c_xmm = evstr(FuncName2+"(xmm,Params)");

//    	    valuepp = feval(FuncName,xpp) + lam(:)'*c_xpp(:);
            valuepp = evstr(FuncName+"(xpp,Params)")+ lam(:)'*c_xpp(:); 
//	    valuepm = feval(FuncName,xpm) + lam(:)'*c_xpm(:);
            valuepm = evstr(FuncName+"(xpm,Params)")+ lam(:)'*c_xpm(:);
//	    valuemp = feval(FuncName,xmp) + lam(:)'*c_xmp(:);
            valuemp = evstr(FuncName+"(xmp,Params)")+ lam(:)'*c_xmp(:);
//	    valuemm = feval(FuncName,xmm) + lam(:)'*c_xmm(:);
            valuemm = evstr(FuncName+"(xmm,Params)")+ lam(:)'*c_xmm(:);

	  else

//	    valuepp = feval(FuncName,xpp);
            valuepp = evstr(FuncName+"(xpp,Params)"); 
//	    valuepm = feval(FuncName,xpm);
            valuepm = evstr(FuncName+"(xpm,Params)"); 
//	    valuemp = feval(FuncName,xmp);
            valuemp = evstr(FuncName+"(xmp,Params)"); 

//	    valuemm = feval(FuncName,xmm);
            valuemm = evstr(FuncName+"(xmm,Params)"); 

	  end
	  jac(j,i) = (valuepp-valuepm-valuemp+valuemm)/(4*h_i*h_j);
	  if ( maxiter2 > 1 )
	    if ( nmode > 0 )
	      h_i = h_i/2;
	      h_j = h_j/2;
	    else
	      h_i = h_i*7;
	      h_j = h_j*7;
	    end
	    if ( count > 0 )
	      acc = abs(jac(j,i)-oldvalue);
	      if ( accold < acc )
		if ( nmode > 0 )
		  h_i = h_i*2;
		  h_j = h_j*2;
		else 
		  h_i = h_i/9;
		  h_j = h_j/9;
		end
		nmode = -nmode;
	      end
	    end
	    accold = acc;
	    oldvalue = jac(j,i);
	  end
	  count = count+1;
	end
	if ( maxiter2 > 1 & acc > eps2 & printWarnings > 0 )
	  fprintf([1,'WARNING: Desired accuracy for second' ...
		   ' derivatives (%e) could not be  obtained' ...
		   ' within max number of iterations (%e).' ...
		   ' Final error  estimate is: %e\n'],eps2, ...
		  maxiter2,acc)
	end
      end
    end
  end
  jac = sparse(jac);
  jac = mtlb_sparse(jac);
end
endfunction
