/**********************************************************************
 * $Id$
 * $Source$
 *
 * Part of Scilab Mex interface for IPOPT
 *
 * This code is published under the Common Public License
 * http://www.opensource.org/licenses/cpl.php
 *
 * Copyright (C) 2004; All Rights Reserved
 *
 * Claas Michalik <Michalik<at>LPT<dot>RWTH-Aachen<dot>de>
 * Process Systems Engineering Institute, RWTH Aachen
 *
 * Steinar Hauan <hauan@cmu.edu>
 * Dept of ChemE, Carnegie Mellon University
 * 
 * Modifications to Scilab Interface made by
 * Edson Cordeiro do Valle <edsoncv@vrtech.com.br>
 * Departamento de Desenvolvimento e Tecnologia, VRTech Tecnologias Industriais LTDA.
 * www.vrtech.com.br
 **********************************************************************/  

#include "IpoptMex.h"


/**** global variables holding general information and parameters ***/     
extern fint     G_NoOfConstraints; /* number of constraints         */
extern int      G_debug;           /* print debug info if TRUE      */
extern int      G_printLevel;      /* determines how much info is
                                      printed                       */  
extern int      G_reassurance;     /* determines how intensively
                                      matrices are analysed         */
extern mxArray* G_FD_Params;       /* vector containig parameters 
                                      for the finite differences
                                      derivative calculation        */
extern mxArray* G_Params1;         /* vector containing parameters
				      passed to any MATLAB function */

/**** Global strings holding names of MATLAB callback functions *****/
extern mxArray** G_FuncNames;

/******** Global variables needed to store tracing information ******/
extern bool    G_Trace;
extern int     G_Iterationcount;
extern double* G_XVector;
extern double* G_ObjVector;

/********** Global variables used for sparsity analysis *************/
extern bool*    G_jacobisparse;      /* true if jac. is sparse      */ 
extern bool*    G_hessesparse;       /* true if hesseian is sparse  */ 
extern fint*    G_jacobifixedrow;    /* row-indices of jacobain     */
extern fint*    G_jacobifixedcol;    /* column-indices of jacobian  */
extern fint*    G_hessefixedrow;     /* row-indices of hessian      */
extern fint*    G_hessefixedcol;     /* column-indices of hessain   */
extern int      G_nonzerosjac;       /* #-non zero elements in jac. */      
extern int      G_nonzeroshess;      /* #-non zero elements in hess.*/   
extern mxArray* G_FDJacobianPattern; /* sparsity pattern of Jacobian
					needed for FD-algorithm     */ 
extern mxArray* G_FDHessianPattern;  /* sparsity pattern of Hessian
					needed for FD-algorithm     */

/********* Global variables holding information on bounds ***********/
extern fint*    G_ilb;                /* inidces of lower bounds    */ 
extern fint*    G_iub;                /* indices of upper bounds    */ 
extern fint     G_nlb;                /* number of lower bounds     */ 
extern fint     G_nub;                /* number of upper bounds     */
extern double*  G_bnds_u;             /* values of lower bounds     */
extern double*  G_bnds_l;             /* values of upper bounds     */  

extern char* buf;          /* temporary string storage for messages */
extern char* buf2;         /* temporary string storage for messages */
extern char* tempString;   /* temporary string storage for messages */


/*********************************************************************
 * void flush
 *********************************************************************
 * internal function to flush output
 *********************************************************************/
void flush(){
  fflush(stdout);
  fflush(stderr);
}

/*********************************************************************
 * void MyInfo
 *********************************************************************
 * internal gateway function for information output
 *
 * if the assigned 'print level' for the current message is
 * lower than the current print level (G_printLevel), we print it.
 *
 * typical print levels are:
 *   0: Print nothing (just ERRORS)
 *   1: Just print warnings
 *   5: We usually want this (prints a few additional infos) [default]
 *  10: We sometimes want this (prints some additional informations)
 * 100: We usually DONT want this
 *********************************************************************/
void MyInfo(char* msg, int msg_lvl){
  if ( msg_lvl <= G_printLevel ) {
    flush();
    if ( msg_lvl == 1 )
      mexPrintf("WARNING: ");
    mexPrintf(msg);
    flush();
  }
}

/*----------------------------------------------------------------------
  this function returns a timestamp from gettimeofday()
  with the seconds and microseconds "joined" into a single variable.

  this is only the wall clock time -- NOT cpu time.
--------------------------------------------------------------------*/

long double GetTimeStamp(void){
  long double r = 0;

#ifdef __UNIX__
  struct timeval tv;

  if ( gettimeofday(&tv,NULL) ){
    printf("gettimeofday() failed: errno=%d\n",errno);
  } else {
    r = tv.tv_sec + tv.tv_usec /((double) 1000000);
  }
#else
  time_t t = time(NULL);
  r = (long double) t;
#endif

  return(r);
}

/********************************************************************* 
 * void MyDebug
 ********************************************************************
 * internal gateway function for DEBUG output
 *
 * if the assigned 'debug level' for the current message is
 * lower than the current debug level (G_debug), we print it.
 *
 * typical debug levels are:
 *   1: we "almost always" want this
 *  10: we usually want this
 * 100: we usually DONT want this
 *********************************************************************/
void MyDebug(char *msg, int msg_lvl){
  if ( msg_lvl <= G_debug ) {
    flush();
    mexPrintf(msg);
    flush();
  }
}

/*********************************************************************
 * void MyIpoptError
 *********************************************************************
 * Internal gateway function for ERROR output
 *********************************************************************/
void MyIpoptError(char* msg){
  char* str ;
  str = (char *) mxCalloc(512,sizeof(char));
  flush();
  sprintf(str,"\nE R R O R: %s\n",msg);
  mexErrMsgTxt(str);
  flush();
  mxFree(str);
}


/*********************************************************************
 * fint* CreateFintVector
 *********************************************************************
 * Function returns a vector of fint's of length 'Capacity' or gives
 * an error message if it cannot allocate memory. The first element of
 * the array holds the capacity  
 ********************************************************************/
fint* CreateFintVector(int Capacity) {
  fint* Vector;
  if ( Capacity <= 0 )
    MyInfo("CreateFintVector was called with Capacity less or equal\
            to zero, no vector will be created !",1);
  else{
    if ( (Vector = (fint *)mxCalloc(Capacity,sizeof(fint))) == NULL ) {  
      sprintf(buf,"Could not allocate memory, \
                   calloc returns error no: %d",errno);  
      MyIpoptError(buf);
    }
    else {
      *(Vector) = (fint) Capacity;
      return Vector;
    }
  }
  return NULL;
}


/*********************************************************************
 * double* CreateDoubleVector
 *********************************************************************  
 * Function returns a vector of double's with length 'Capacity' or 
 * gives an error message if it cannot allocate memory. The first
 * element of the array holds the capacity  
 *********************************************************************/  
double* CreateDoubleVector(int Capacity) {
  double* Vector;
  if(Capacity <= 0)
    MyInfo("CreateDoubleVector was called with Capacity less or equal\
to zero, no vector will be created !",1);
  else{
    if ( (Vector=(double *)mxCalloc(Capacity,sizeof(double) ))== NULL) {    
      sprintf(buf,"Could not allocate memory, calloc returns error no:\
 %d",errno);  
      MyIpoptError(buf);
    }
    else {
    *(Vector) = Capacity;
    return Vector;
    }
  }
  return NULL;
}


/********************************************************************
 * double* insertZeroAt
 ********************************************************************
 * This function inserts a 0 into the passed by reference vector of
 * doubles at the given position, this means the rest of the vector
 * will be moved towards the end
 *******************************************************************/        
double* insertZeroAt(double* vector,int newlength, int position) {
  double* temp = CreateDoubleVector(newlength);
  memcpy(temp,vector,newlength*sizeof(double));
  vector[position] = (double) 0;
  memcpy(vector+position+1, temp+position,
	 (newlength-position-1)*sizeof(double));       
  mxFree(temp);
  return NULL;
}
	 

/********************************************************************
 * double* AddtoVector
 ********************************************************************  
 * Adds 'howMany' variables from 'Values' to 'Vector' makes room, if 
 * 'Vector' is too small  
 ********************************************************************/
double* AddToVector(int howMany, double *Vector, double *Values) { 
  int     maxlength       = (int) *Vector;
  int     necessaryLength = 1+(G_Iterationcount+1)*howMany;
  int     newmaxlength;
  int     startIndex;
  double* tempVector;

  if ( !Vector || howMany <= 0 || maxlength <= 0 ){
    mexPrintf("AddToVector(): internal error -- please report bug.\n");
  }

  MyDebug("\nAt start of AddToVector\n",1);
  if ( necessaryLength > maxlength ) {

    /* find an binary factor by which to increase maxlength */
    newmaxlength = maxlength;
    while ( newmaxlength < necessaryLength )
      newmaxlength *= 2;

    tempVector   = CreateDoubleVector(newmaxlength);   
    memcpy(tempVector, Vector, maxlength*sizeof(double));

    /* free old memory to avoid leaking it */
    if ( Vector )
      mxFree(Vector);

    /* reassign pointer to new location */
    Vector       = tempVector;
    *(Vector)    = newmaxlength;
  }
  startIndex     = 1+G_Iterationcount*howMany;
  memcpy(Vector+startIndex, Values, howMany*sizeof(double));
  MyDebug("Leaving AddToVector\n",1);
  return Vector; 
}


/*************************************************************
 * double RandomNumber
 ************************************************************* 
 * Returns a random double with min <= random double <= max  
 *************************************************************/
double RandomNumber(double min, double max) {
  double random = ( (double)rand()/(double)RAND_MAX );
  return (random*(max-min)+min);
}


/************************************************************ 
 * void* setRandomValidPoint
 ************************************************************
 * This functions sets _point to a valid random point 
 ************************************************************/  
void* setRandomValidPoint(double* _point,int _n) {
  int i = 0;
  double* lowerbounds = CreateDoubleVector(_n);
  double* upperbounds = CreateDoubleVector(_n);

  /* initialize: lower bounds to -INF, upper bounds to +INF */
  for (i=0; i<_n ; i++) { 
    lowerbounds[i] = (double) -INF;
    upperbounds[i] = (double) +INF;
  }

  /* set provided lower bounds to correct values */
  for (i=0; i < G_nlb; i++) 
    lowerbounds[G_ilb[i]-1] = G_bnds_l[i];

  /* set provided upper bounds to correct values */
  for (i=0; i < G_nub; i++) 
    upperbounds[G_iub[i]-1] = G_bnds_u[i];

  for (i=0; i < _n; i++)
    _point[i] = RandomNumber(lowerbounds[i],upperbounds[i]);

  mxFree(lowerbounds);
  mxFree(upperbounds);
  return (NULL);
}


/**********************************************************************
 * void setPRHS
 **********************************************************************
 * This function allocates memory and sets up the rhs-mxArray for the
 * MATALB callback  depending on the method:
 *         -1 = Calculate Function (given as .m-file)
 *          0 = Calculate Gradient of the Objective using ADMAT
 *          1 = Calculate Jacobian of the Constraints using ADMAT
 *          2 = Calculate Hessian of Lagrange Function using ADMAT
 *         10 = Calculate Gradient of the Objective using finite
 *              differences, simple algorithm
 *         11 = Calculate Jacobian of the Objective using finite
 *              differences, complex algorithm
 *         20 = Calculate the full Jacobian of the constraints using
 *              finite differences, simple algorithm
 *         21 = Calculate the full Jacobian of the constraints using
 *              finite differences, complex algorithm
 *         22 = Calculate the sparse Jacobian of the constraints using
 *              finite differences, simple algorithm
 *         23 = Calculate the sparse Jacobian of the constraints using  
 *              finite differences, complex algorithm
 *         30 = Calculate the full Hessian of the Lagrangian using
 *              finite differences
 *         31 = Calculate the sparse Hessian of the Lagrangian using
 *              finite differences
 ************************************************************************/
void setPRHS(int _nrhs,int _rowsrhs,int _colsrhs,double* _x,double*
	     _lambda,int _n,int _NoOfCons,int _method,mxArray**
	     L_prhs, int _task  ) { 
  sprintf(buf,"\nAt start of setPRHS with method = %d, rowsrhs = %d,\
               colsrhs = %d nrhs = %d",_method,_rowsrhs,_colsrhs,_nrhs); 
  MyDebug(buf,1);

  /* prhs[0] and prhs[1] (= point x and parameter-vector) are the same
   * for all methods */ 
  L_prhs[0] = mxCreateDoubleMatrix(_n,1,mxREAL);
  memcpy(mxGetPr(L_prhs[0]),_x,_n*sizeof(double));
  L_prhs[1] = (G_Params1);

  if (_method == -1) {
    if (_nrhs > 2) {
      L_prhs[2] = mxCreateDoubleMatrix(_NoOfCons,1,mxREAL);
      memcpy(mxGetPr(L_prhs[2]),_lambda,_NoOfCons*sizeof(double));
    }
  }
  else if (_method == 0 || _method == 1) { /* AD Obj.-Grad/Jacobian */
    L_prhs[2] = mxCreateDoubleScalar(0.0);
    L_prhs[3] = mxCreateDoubleScalar(_method);
    L_prhs[4] = (G_FuncNames[_method]);
  }
  else if (_method == 2) { /* AD Hessian sparse */
    L_prhs[2] = mxCreateDoubleMatrix(_NoOfCons,1,mxREAL);
    memcpy(mxGetPr(L_prhs[2]),_lambda,_NoOfCons*sizeof(double));
    L_prhs[3] = mxCreateDoubleScalar(2.0);
    L_prhs[4] = (G_FuncNames[0]);
    L_prhs[5] = (G_FuncNames[1]);
	/* This is a dummy assignment, due to historical reasons when 
	admat was used instead of ADiMat */
    L_prhs[6] = (G_FuncNames[0]);
  }
  else if (_method == 10 || _method == 11) { /* FD obj.Grad. full */
    L_prhs[2] = mxCreateDoubleScalar(0.0);
    L_prhs[3] = (G_FuncNames[0]);
    L_prhs[4] = (G_FuncNames[1]);
    L_prhs[5] = (G_FD_Params);
    L_prhs[6] = mxCreateDoubleScalar(1.5);
    L_prhs[7] = mxCreateDoubleScalar(_method);
  }
  else if (_method >= 20 && _method <= 23) { /*FD Jac. sparse/full*/
    L_prhs[2] = mxCreateDoubleScalar(0.0);
    L_prhs[3] = (G_FuncNames[1]);
    L_prhs[4] = (G_FuncNames[1]);
    L_prhs[5] = (G_FD_Params);
    if (_task == 0)
      L_prhs[6] = mxCreateDoubleScalar(1.5);
    else 
      L_prhs[6] = (G_FDJacobianPattern);
    L_prhs[7] = mxCreateDoubleScalar(_method);
  }
  else if (_method == 30 || _method == 31) { /*FD Hess. sparse/full*/
    L_prhs[2] = mxCreateDoubleMatrix(_NoOfCons,1,mxREAL);
    memcpy(mxGetPr(L_prhs[2]),_lambda,_NoOfCons*sizeof(double));
    L_prhs[3] = (G_FuncNames[0]);
    L_prhs[4] = (G_FuncNames[1]);
    L_prhs[5] = (G_FD_Params);
    if (_task == 0)
      L_prhs[6] = mxCreateDoubleScalar(1.5);
    else
      L_prhs[6] = (G_FDHessianPattern);
    L_prhs[7] = mxCreateDoubleScalar(_method);
  }
  MyDebug("\nAt end of setPRHS",1);
}


/********************************************************************* 
 * int destroyPRHS 
 *********************************************************************
 * This function destroys all created prhs-Arrays depending on the
 * method (see above for a list of: number --> method)
 *
 * Aug 23 2005, SHa
 *
 *   this function probably creates memory leaks by setting the
 *   RHS of mxArrays to NULL and thus losing the pointer.
 *
 *   exactly how to deal with this will be determined later; for now
 *   it is not considered to be a major issue.
 *********************************************************************/ 
int destroyPRHS(mxArray **prhs, int _method, int _nrhs, 
		int _NoOfCons, int _task) 
{ 
  MyDebug("\nAt start of destroyRHS",1);
  mxDestroyArray(prhs[0]);

  (prhs[1])=NULL;

  if ( _method == -1 ) {
    if (_nrhs > 2) 
       mxDestroyArray(prhs[2]);
  }
  else if (_method == 0 || _method == 1) { /* AD Obj.Grad/Jacobian */
    mxDestroyArray(prhs[2]);
    mxDestroyArray(prhs[3]);
    (prhs[4])=NULL;
  }
  else if ( _method == 2 ) { /* AD Hessian sparse */
    mxDestroyArray(prhs[2]);
    mxDestroyArray(prhs[3]);
    (prhs[4])= NULL;
    (prhs[5])= NULL;
    (prhs[6])= NULL;
  }
  else if ( _method >= 10 || _method <= 31 ) { /* FD obj.Grad. full */
    mxDestroyArray(prhs[2]);
    (prhs[3])=NULL;
    (prhs[4])=NULL;
    (prhs[5])=NULL;
    if ( _task != 0 )
      (prhs[6])=NULL;
    mxDestroyArray(prhs[6]);
    mxDestroyArray(prhs[7]);
  }
  MyDebug("\nAt end of destroyPRHS",1);
  return 1;
}


/*********************************************************************   
 * int InitSparseMatrixPattern
 *********************************************************************
 * This function gets called if a sparse matrix is returned by a
 * MATLAB function. 
 * It sets up the vectors containing the row- and colum indices for
 * the Hessian or the Jacobian,  needed by IPOPT.
 * It also sets up the vectors G_FDHessianPattern and
 * G_FDJacobianPattern,respectively, needed for the finite differences  
 * algorithm
 *********************************************************************/         
int InitSparseMatrixPattern(int actuallength, int colslhs, int _n,
			    mxArray** plhs,fint* rowindex,fint*
			    colindex, int method)  
{  
  int i = 0, j = 0, k = 0;
  double* SparsityMatrix = NULL;
  sprintf(buf,"\nAt start of InitSparseMatrixPattern with method = %d",
	  method); 
  MyDebug(buf,1);
  for (i=0;i<actuallength;i++)
    rowindex[i] = *(mxGetIr(plhs[0])+i) + 1;

  i=0, j=0, k=0;
  for (j=0; j<colslhs; j++) {
    for (k=0; k < ( *(mxGetJc(plhs[0])+j+1) -
		    (*(mxGetJc(plhs[0])+j)) ); k++) {
      colindex[i] = j+1;
      i++;
    }
  }
  /******** Set up Sparsity Matrix needed for FD-Algorithm ********/
  if (method >= 20 && method < 30) {  /*using finit diff. for jacobian*/
    SparsityMatrix  = CreateDoubleVector(_n * G_NoOfConstraints);
    G_FDJacobianPattern =
      (mxArray *) mxCreateDoubleMatrix(G_NoOfConstraints,_n,mxREAL); 
    for (i=0;i<actuallength;i++) {
      j = (colindex[i]-1)*G_NoOfConstraints + rowindex[i]-1;
      *(SparsityMatrix + j) = (double) 2;
    }
    memcpy(mxGetPr(G_FDJacobianPattern),SparsityMatrix,
	   G_NoOfConstraints*_n*sizeof(double));
    mxFree(SparsityMatrix );
  }
  if (method >= 30) {  /* using finite differences for hessian*/
    SparsityMatrix  = CreateDoubleVector(_n *_n);
    G_FDHessianPattern=(mxArray*) mxCreateDoubleMatrix(_n,_n,mxREAL);   
    for (i=0;i<actuallength;i++) {
      j = (colindex[i]-1)*_n + rowindex[i]-1;
      *(SparsityMatrix + j) = (double) 2;
    }
    memcpy(mxGetPr(G_FDHessianPattern),SparsityMatrix,
	   _n*_n*sizeof(double));
    mxFree(SparsityMatrix);
  }
  MyDebug("\nAt end of InitMatrixPattern",1);
  return 1;
}


/********************************************************************  
 * int InitFullMatrixPattern
 ********************************************************************
 * This function gets called if a full matrix is returned by a MATLAB
 * function. 
 * It sets up the vectors containing the row- and colum indices for
 * the Hessian or the Jacobian,  needed by IPOPT.
 * It also sets up the vectors G_FDHessianPattern and
 * G_FDJacobianPattern, respectively, needed for the finite difference  
 * algorithm
 *********************************************************************/  
int InitFullMatrixPattern(int colslhs, int rowslhs, fint* rowindex,
			  fint* colindex, int _n, int resultlength,
			  int method) { 
  int i = 0, j = 0;
  double* SparsityMatrix = NULL;

  for (i = 0; i < colslhs; i++) {
    for (j = 0; j < rowslhs; j++) {
      rowindex[j+i*rowslhs] = j+1;
      colindex[j+i*rowslhs] = i+1;
    }
  }
  if (method >=20 && method < 30) {  /*using finit diff. for jacobian*/
    SparsityMatrix  = CreateDoubleVector(_n * G_NoOfConstraints);
    G_FDJacobianPattern =
      (mxArray *) mxCreateDoubleMatrix(G_NoOfConstraints,_n,mxREAL);  
    for (i=0; i<resultlength; i++) {
      j = (colindex[i]-1)*G_NoOfConstraints + rowindex[i]-1;
      *(SparsityMatrix + j) = (double) 2;
    }
    memcpy(mxGetPr(G_FDJacobianPattern),SparsityMatrix,
             G_NoOfConstraints*_n*sizeof(double));
    mxFree(SparsityMatrix );
  }
  if ( method >= 30) {  /* using finite differences for hessian*/
    SparsityMatrix  = CreateDoubleVector(_n*_n);
    G_FDHessianPattern=(mxArray*) mxCreateDoubleMatrix(_n,_n,mxREAL);  
    for (i=0; i<resultlength; i++) {
      j = (colindex[i]-1)*_n + rowindex[i]-1;
      *(SparsityMatrix + j) = (double) 2;
    }
    memcpy(mxGetPr(G_FDHessianPattern),SparsityMatrix,
	   _n*_n*sizeof(double));
    mxFree(SparsityMatrix);
  }
  return 1;
}


/*******************************************************************
 * int FillInExtraZeros
 *******************************************************************
 * This function fills in extra zeros into the passed by reference
 * result vector if necessary (IPOPT expects this vector to have the
 * same length every time, but MATLABS sparse matrix vector change
 * their size depending on the number of non zero elements).
 *******************************************************************/
int FillInExtraZeros(int actuallength, int maxlength, int colslhs,
		     mxArray** plhs, double* result, 
		     char *now_evalfname) {
  int i, j, k, found = 0, eval_findex = -1, STRLEN = 0;
  char *g_evalfname;
  fint *testcolindex = NULL;
  fint *testrowindex = NULL;

  MyDebug("\nAt start of FillInExtraZeros",1);
  testcolindex = CreateFintVector(actuallength);
  testrowindex = CreateFintVector(actuallength);

  for (i=0;i<actuallength;i++) {
    testrowindex[i] = *(mxGetIr(plhs[0])+i) + 1;
  }

  i=0, j=0, k=0;
  for (j=0; j<colslhs; j++) {
    for (k=0; k < ( *(mxGetJc(plhs[0])+j+1) -
		    (*(mxGetJc(plhs[0])+j)) ); k++) {
      testcolindex[i] = j+1;
      i++;
    }
  }

  g_evalfname = (char*) mxCalloc(BUFLEN,sizeof(char));

  for (i=0; i<5; i++){
    STRLEN = MSIZE(G_FuncNames[i]) * sizeof(mxChar) + 1;
    mxGetString(G_FuncNames[i],g_evalfname,STRLEN*sizeof(mxChar));
    if ( strcmp(now_evalfname,g_evalfname) == 0 ){
      eval_findex = i;
      break;
    }
  }

  switch(eval_findex){
  case 4: /* Hessian Matrix */
    MyDebug("Now evaluting the Hessian Matrix.\n",1);
    for (i=0;i<maxlength;i++) {
      found = 0;
      j = 0;
      while ( j < actuallength && j <= i ) {
	if ( G_hessefixedrow[i] == testrowindex[j]
	     && G_hessefixedcol[i] == testcolindex[j] )
	  found = 1;
	j++;
      }
      if ( !found ) { /* fill in an extra zero */
	insertZeroAt(result,maxlength,i);
      }
    }
    break;
    
  case 3: /* Jacobian Matrix */
    MyDebug("\nNow evalute the Jacobian Matrix.\n",1);
    for (i=0;i<maxlength;i++) {
      found = 0;
      j = 0;
      while ( j < actuallength && j <= i ) {
	if ( G_jacobifixedrow[i] == testrowindex[j]
	     && G_jacobifixedcol[i] == testcolindex[j])
	  found = 1;
	j++;
      }
      if ( !found ) { /* fill in an extra zero */
	insertZeroAt(result,maxlength,i);
      }
    }
    break;
  default:
    break;
  }

  mxFree(testcolindex);
  mxFree(testrowindex);
  mxFree(g_evalfname);

  MyDebug("\nAt end of FilInExtraZeros.\n",1);
  return 1;
}

/***********************************************************************
 * int CallMatlabSparse
 ***********************************************************************  
 * Function for MATLAB callback if the MATLAB function returns a
 * sparse matrix; the purposes of the variables are :
 *
 * _nzmax    Maximum number of nonzeros in matrix
 * _n        Number of variables in problem
 * nrhs      Number of right hand side arguments
 * nlhs      Number of left hand side arguments
 * _x        Point to evaluate at this iteration 
 * result    Result
 * rwoslhs   Number of rows for first left hand side argument
 * colslhs   Number of columns for first left hand side arg
 * rwosrhs   Number of rows for first right hand side arg 
 * colsrhs   Number of columns for first right hand side arg
 * resultlength 
 *           Number of elements in the MATLAB function result
 *           (= number of elements to copy) 
 * _matlabFuncName 
 *           Name of the MATLAB function to call
 * _lambda   Values of lambda
 * rowindex  Vector of row indices of the matrix 
 * colindex  vector of column indices of the matrix
 * maxlength Maximal number of non zero elements in matrix
 * task      For task = 0 some initializations take place
 * method    detemines the method used to calculate function values:
 *           -1 = Calculate Function (given as .m-file)
 *            0 = Calculate Gradient of the Objective using ADMAT
 *            1 = Calculate Jacobian of the Constraints using ADMAT
 *            2 = Calculate Hessian of Lagrange Function using ADMAT
 *           10 = Calculate Gradient of the Objective using finite
 *                differences, simple algorithm
 *           11 = Calculate Jacobian of the Objective using finite
 *                differences, complex algorithm
 *           20 = Calculate the full Jacobian of the constraints using 
 *                finite differences, simple algorithm
 *           21 = Calculate the full Jacobian of the constraints using 
 *                finite differences, complex algorithm
 *           22 = Calculate the sparse Jacobian of the constraints
 *                using finite differences, simple algorithm
 *           23 = Calculate the sparse Jacobian of the constraints
 *                using finite differences, complex algorithm
 *           30 = Calculate the full Hessian of the Lagrangian using
 *                finite differences
 *           31 = Calculate the sparse Hessian of the Lagrangian using 
 *                finite differences
 ************************************************************************/ 
int CallMatlabSparse(int _nzmax,int _n,int nrhs,int nlhs,double*
                     _x,double* result,int rowslhs,int colslhs,int
                     rowsrhs, int colsrhs,int resultlength, char* 
                     _matlabFuncName, double* _lambda,fint*
                     rowindex,fint* colindex, int maxlength, 
		     int task, int method)  
{
  int       i             = 0;
  int       actuallength  = 0;
  double*   zeros         = NULL;
  mxArray** plhs          = NULL; 
  mxArray** Sparse_prhs   = NULL; 
  mxArray** numOfElements = NULL;

  flush();
  sprintf(buf,"At start of CallMatalabSparse, \
              method = %i nrhs = %i\n",method,nrhs);
  MyDebug(buf,1);

  plhs          = (mxArray**) mxCalloc(1, sizeof(mxArray*));
  numOfElements = (mxArray**) mxCalloc(1, sizeof(mxArray*));

  /******** create Sparse_prhs for MATLAB Callback ********/
  Sparse_prhs   = (mxArray**) mxCalloc(10,sizeof(mxArray*));
  setPRHS(nrhs, rowsrhs, colsrhs, _x, _lambda, _n,
	  G_NoOfConstraints, method, Sparse_prhs, task); 

  /*********** create plhs for MATLAB callback ************/
  plhs[0] = mxCreateSparse(rowslhs,colslhs,_nzmax,mxREAL);
  memcpy(mxGetPr(Sparse_prhs[0]),_x,_n*sizeof(double));
  numOfElements[0] = mxCreateDoubleScalar(0.0); 
  zeros = CreateDoubleVector(maxlength);

  mexCallMATLAB(1,plhs         ,nrhs,Sparse_prhs,_matlabFuncName); 
  mexCallMATLAB(1,numOfElements,1   ,plhs       ,"nnz"          ); 
  actuallength = (int) mxGetScalar(numOfElements[0]);

  if ( task != 0 ) {
    memcpy( result, zeros           , resultlength*sizeof(double) ); 
    memcpy( result, mxGetPr(plhs[0]), actuallength*sizeof(double) );
  }
  /**************** Initialize matrix structure ********************/  
  if (task == 0) {
    i = InitSparseMatrixPattern(actuallength, colslhs, _n, plhs,
				rowindex, colindex, method); 
    if (i == 0)
      MyIpoptError("Could not initialize matrix pattern");

  } else { /* task != 0 */

    if ( actuallength > maxlength ) {
      /* Give error message and quit if more non zero elements occure 
       * than the maximum number of non zero elements epected */
      sprintf(buf,"MATLAB function %s returned %d number of non zero elements!\n\
                   The max number of non zero elements expected is %d.\n\
                   You can address this problem by setting parameter\
                   findnonzeros to a higher value.",
	      _matlabFuncName, actuallength, maxlength); 
      MyIpoptError(buf);
    } else { /* task != 0 && actuallength <= maxlength */
      
      memcpy( result, zeros , resultlength*sizeof(double));
      memcpy( result, mxGetPr(plhs[0]), actuallength*sizeof(double));

      /*
       * fill in extra zeros of sparse matrix returned from MATLAB
       * happen to have fewer non-zero elements than usual
       */
      if ( actuallength > 0 && actuallength < maxlength ){
	i = FillInExtraZeros(actuallength,maxlength,
			     colslhs,plhs,result,_matlabFuncName);
	if (i != 1)
	  MyIpoptError("Could not fill in extra zeros");
      }
    }
  }
  flush();
  /******************** free allocated memory *********************/ 
  mxDestroyArray(plhs[0]);
  mxDestroyArray(numOfElements[0]);
  mxFree(plhs);
  mxFree(numOfElements);
  destroyPRHS(Sparse_prhs,method,nrhs,G_NoOfConstraints,task);
  mxFree(Sparse_prhs);
  mxFree(zeros);
  MyDebug("Leaving CallMatlabSparse\n",1);
  return actuallength;
}


/**********************************************************************
 * int CallMatlabFull
 **********************************************************************
 * Function for MATLAB callback if the MATLAB function returns a sparse
 * matrix; the purposes of the variables are :
 *
 * _nzmax    Maximum number of nonzeros in matrix
 * _n        Number of variables in problem
 * nrhs      Number of right hand side arguments
 * nlhs      Number of left hand side arguments
 * _x        Point to evaluate at this iteration 
 * result    Result
 * rwoslhs   Number of rows for first left hand side argument
 * colslhs   Number of columns for first left hand side argument
 * rwosrhs   Number of rows for first right hand side argument
 * colsrhs   Number of columns for first right hand side argument
 * resultlength 
 *           Number of elements in the MATLAB function result
 *           (= number of elements to copy)
 * _matlabFuncName 
 *           Name of the MATLAB function to call
 * _lambda   Values of lambda
 * rowindex  Vector of row indices of the matrix 
 * colindex  vector of column indices of the matrix
 * task      For task = 0 some initializations take place
 * method    detemines the method used to calculate function values:
 *           -1 = Calculate Function (given as .m-file)
 *            0 = Calculate Gradient of the Objective using ADMAT
 *            1 = Calculate Jacobian of the Constraints using ADMAT
 *            2 = Calculate Hessian of Lagrange Function using ADMAT
 *           10 = Calculate Gradient of the Objective using finite
 *                differences, simple algorithm
 *           11 = Calculate Jacobian of the Objective using finite
 *                differences, complex algorithm
 *           20 = Calculate the full Jacobian of the constraints using
 *                finite differences, simple algorithm
 *           21 = Calculate the full Jacobian of the constraints using
 *                finite differences, complex algorithm
 *           22 = Calculate the sparse Jacobian of the constraints
 *                using finite differences, simple algorithm
 *           23 = Calculate the sparse Jacobian of the constraints
 *                using finite differences, complex algorithm
 *           30 = Calculate the full Hessian of the Lagrangian using
 *                finite differences
 *           31 = Calculate the sparse Hessian of the Lagrangian using
 *                finite differences
 ************************************************************************/
int CallMatlabFull(int _nzmax, int _n, int nrhs, int nlhs, double* _x,
		   double* result,int rowslhs,int colslhs,int
		   rowsrhs,int colsrhs,int resultlength,char*
		   _matlabFuncName,double* _lambda,fint*
		   rowindex,fint* colindex,int task, int method)        
{
  int       i         = 0;
  mxArray** plhs      = NULL;
  mxArray** Full_prhs = NULL;

  sprintf(buf,"\nAt start of CallMatlabFull, method = %i \n",method);   
  MyDebug(buf,1);

  /******** Create and set Full_prhs for MATLAB callback ************/ 
  Full_prhs = (mxArray**) mxCalloc(10,sizeof(mxArray*));
  setPRHS(nrhs, rowsrhs, colsrhs, _x, _lambda, _n, G_NoOfConstraints,
	  method, Full_prhs, task); 

  /********** Create and set plhs for MATLAB callback **************/
  plhs = (mxArray**) mxCalloc(1, sizeof(mxArray*));
  plhs[0] = mxCreateDoubleMatrix(rowslhs,colslhs,mxREAL);
  mexCallMATLAB(1, plhs, nrhs, Full_prhs, _matlabFuncName);

  if ( mxIsSparse(plhs[0]) ) {
    sprintf(buf,"Function %s returned a sparse matrix, but is\
                 supposed to return a full matrix",_matlabFuncName);
    MyIpoptError(buf);
  } 
  if (mxGetNumberOfElements(plhs[0]) != rowslhs*colslhs) {
    sprintf(buf,"Function %s returned a matrix of wrong size\n",
	    _matlabFuncName);
    sprintf(buf2,"\nSuppodes number of elements in matrix: %d\
                  returned number of elements:%d",
	    rowslhs*colslhs,mxGetNumberOfElements(plhs[0]));  
    strcat(buf,buf2);
    MyIpoptError(buf);
  }
  memcpy(result,mxGetPr(plhs[0]),resultlength*sizeof(double));

  if (colslhs > 1 && task == 0) {
    i = InitFullMatrixPattern(colslhs, rowslhs, rowindex, colindex,
			      _n, resultlength, method); 
    if(i != 1)
      MyIpoptError("\nCould not create Pattern for full Matrix");
  }
  flush();
  /*************** Free memory ***************************/
  mxDestroyArray(plhs[0]);
  mxFree(plhs);
  destroyPRHS(Full_prhs,method,nrhs,G_NoOfConstraints,task);
  mxFree(Full_prhs);
  MyDebug("Leaving CallMatlabFull\n",1);
  return 0;
}


/********************************************************************
 * int AnalyzeSparseMatrix
 ********************************************************************
 * This function aims to find the maximum number of non zero
 * elements returned by a MATLAB function and calls the function
 * CallMatlabSparse with task == 0 in order to initialize the vectors 
 * containing the row- and column indices (these vectors are required
 * by IPOPT)  
 ********************************************************************/   
int AnalyzeSparseMatrix(int _rows, int _cols, int _n, int _nrhs, char*
			_name, int _method, int _reassurance, int _m,
			bool CalcJacobian, bool CalcHessian) { 
  int       nonzerosfound      = 0;
  int       j                  = 0;
  int       STRLEN             = 0;
  double*   dummy              = NULL;
  double*   testlambda         = NULL;
  double*   testx              = NULL;
  double*   besttestx          = NULL;
  mxArray** plhs               = NULL;
  mxArray** AnalyzeSparse_prhs = NULL;
  mxArray** numOfElements      = NULL;

  MyDebug("\nAt start of AnlalyzeSparseMatrix",1);

  if (_m > 0)
    testlambda      = CreateDoubleVector(_m);
  testx             = CreateDoubleVector(_n);
  besttestx         = CreateDoubleVector(_n);
  plhs              = (mxArray**) mxCalloc(1 ,sizeof(mxArray*));
  AnalyzeSparse_prhs= (mxArray**) mxCalloc(10,sizeof(mxArray*));
  numOfElements     = (mxArray**) mxCalloc(1 ,sizeof(mxArray*));
  plhs[0]           = mxCreateSparse(_rows,_cols,_rows*_cols,mxREAL);
  numOfElements[0]  = mxCreateDoubleScalar(0.0);

  /* arbitrarily initialize all lambda values to 1.2 */
  for (j=0; j<_m; j++) {
    testlambda[j] = 1.2;
  }

  /************ finding maximum number of non zero elements *********/  
  sprintf(buf,"\nAnalyzing sparse Matrix from %s and finding non zero\
                 elements ...\n",_name);

  MyInfo(buf,10);
  j = 0;

  while (j < _reassurance) {
    setRandomValidPoint(testx,_n);
    setPRHS(_nrhs, _rows, _cols, testx, testlambda, _n,
	    G_NoOfConstraints, _method,AnalyzeSparse_prhs,0);        
    flush();
    mexCallMATLAB(1,plhs,_nrhs,AnalyzeSparse_prhs,_name);	
    flush();
    mexCallMATLAB(1,numOfElements,1,plhs,"nnz");
    flush();
    if ( (int) mxGetScalar(numOfElements[0]) > nonzerosfound ) {
      nonzerosfound = (int) mxGetScalar(numOfElements[0]);
      j = 0;
      memcpy( besttestx,testx,_n*sizeof(double) );
    }
    else 
      j++;
    destroyPRHS(AnalyzeSparse_prhs,_method,_nrhs,G_NoOfConstraints,0);
  }

  /******************************** free memory ******************/
  mxFree(testx);
  mxDestroyArray(plhs[0]);
  mxDestroyArray(numOfElements[0]);
  mxFree(plhs);
  mxFree(numOfElements);
  mxFree(AnalyzeSparse_prhs);


  /* initialize vectors with row and column indices */
  sprintf(buf,"Number of non zero elements found: %d\n",
	  nonzerosfound);  

  MyInfo(buf,10);
  STRLEN =  mxGetN( G_FuncNames[4] ) * sizeof(mxChar) + 1;
  mxGetString(G_FuncNames[4],tempString,STRLEN);	

  if ( strcmp(_name,tempString) == 0 && CalcHessian == true ) { 
    if (nonzerosfound == 0)
      MyIpoptError("\nSparse Hessian is full; use full method instead"); 
    else {
      G_hessefixedrow = CreateFintVector(nonzerosfound);
      G_hessefixedcol = CreateFintVector(nonzerosfound);
    }     
    CallMatlabSparse(_n*_n,_n,_nrhs,1,besttestx,dummy,_n,_n,_n, 
		     _n,0,_name,testlambda,G_hessefixedrow,
		     G_hessefixedcol,_rows*_cols,0,_method);
  }
  STRLEN =  mxGetN( G_FuncNames[3] ) * sizeof(mxChar) + 1;
  mxGetString(G_FuncNames[3],tempString,STRLEN);	

  if ( strcmp(_name,tempString) == 0 && CalcJacobian == true ) {
    if (nonzerosfound == 0)
      MyIpoptError("\nSparse Jacobian is full; use full method instead");
    else {
      G_jacobifixedrow = CreateFintVector(nonzerosfound);
      G_jacobifixedcol = CreateFintVector(nonzerosfound);
    }    
    CallMatlabSparse(_n*_m,_n,_nrhs,1,besttestx,dummy,_n,_n,_n,
		     _m,0,_name,testlambda,G_jacobifixedrow,
		     G_jacobifixedcol,_rows*_cols,0,_method);
  }

  /*************** Free allocated memory **********************/ 
  if ( _m > 0)
    mxFree(testlambda);  
  mxFree(besttestx);
  MyDebug("Leaving AnalyzeMatrix\n",1);
  return nonzerosfound;
}


/********************************************************************* 
 *  Analyzes the given matrix. Checks whether or not matrix is sparse
 * and finds number of non zero elements. The Variables are:
 * 
 *  1) _n           = number of variables in problem
 *  2) _m           = number of equality constraints
 *  3) _rows        = number of rows in matrix
 *  4) _cols        = number of cols in matrix
 *  5) _nrhs        = number of rhs arguments for MATLAB callback
 *  6) _reassurance = this number specifies, how the number of non
 *                    zero elements is found. The number of non zero
 *                    elements for a random evaluation point has to be
 *                    less or equal to the highest number of non zero
 *                    elements found so far 'reassurance' times in a
 *                    row to be approved. 
 *                    Higher numbers make sure no non zero element is
 *                    missed but need more function evaluations. 
 *  7) _name        = name of the MATLAB function to call
 *  8) _isSparse    = set to sparse, if matrix is sparse
 *  9) method       = determines the method used to calculate matrix
 *           -1 = Calculate Function (given as .m-file)
 *            0 = Calculate Gradient of the Objective using ADMAT
 *            1 = Calculate Jacobian of the Constraints using ADMAT
 *            2 = Calculate Hessian of Lagrange Function using ADMAT
 *           10 = Calculate Gradient of the Objective using finite
 *                differences, simple algorithm
 *           11 = Calculate Jacobian of the Objective using finite
 *                differences, complex algorithm
 *           20 = Calculate the full Jacobian of the constraints using
 *                finite differences, simple algorithm
 *           21 = Calculate the full Jacobian of the constraints using
 *                finite differences, complex algorithm
 *           22 = Calculate the sparse Jacobian of the constraints using
 *                finite differences, simple algorithm
 *           23 = Calculate the sparse Jacobian of the constraints using
 *                finite differences, complex algorithm
 *           30 = Calculate the full Hessian of the Lagrangian using
 *                finite differences
 *           31 = Calculate the sparse Hessian of the Lagrangian using
 *                finite differences
 **********************************************************************/
int AnalyzeMatrix(int _n, int _m, int _rows, int _cols, int _nrhs, int
		  _reassurance, char *_name, bool* _isSparse, int _method) { 
  bool      CalcJacobian   = true;
  bool      CalcHessian    = true;
  int       i              =  0  ;
  int       STRLEN         =  0  ;
  int       returnval      = -1  ;
  double*   dummy          = NULL;
  double*   testlambda     = NULL;
  double*   testx          = NULL;
  double*   besttestx      = NULL;
  mxArray** plhs           = NULL;
  mxArray** analyze_prhs   = NULL;  
  mxArray** numOfElements  = NULL;

  sprintf(buf,"\nAt start of AnalyzeMatrix, method = %i nrhs =%i \n",_method,_nrhs); 
  MyDebug(buf,1);

  if (_m > 0)
    testlambda  = CreateDoubleVector(_m);
  testx         = CreateDoubleVector(_n);
  besttestx     = CreateDoubleVector(_n);
  plhs          = (mxArray**) mxCalloc(1, sizeof(mxArray*));
  numOfElements = (mxArray**) mxCalloc(1, sizeof(mxArray*));
  analyze_prhs  = (mxArray**) mxCalloc(10,sizeof(mxArray*));

  if (_method == 2 || _method >= 30)
    CalcJacobian = false;
  if (_method >= 0 && _method <= 23 && _method != 2)
    CalcHessian  = false;
  
  /* Allocate memory for a sparse matrix with _rows*_cols 
   * non zero elements to make sure enough memory is allocated */ 
  plhs[0] = mxCreateSparse(_rows,_cols,_rows*_cols,mxREAL);

  /* number of non zero elements */
  numOfElements[0] = mxCreateDoubleScalar(0);
  for (i=0; i<_m; i++) {
    /* set all lambda values (arbitrarily) to 1.2 */
    testlambda[i] = 1.2; 
  }
  
  /* set testx to a random valid point */
  setRandomValidPoint(testx,_n);
  /* create appropriate analyze_prhs for MATLAB callback */
  setPRHS(_nrhs,_rows,_cols,testx,testlambda,_n,G_NoOfConstraints,_method,analyze_prhs,0);
  /* check whether or not matrix is sparse */
  flush();

  mexCallMATLAB(1,plhs,_nrhs,analyze_prhs,_name);
  flush();

  if (mxGetM(plhs[0]) != _rows || mxGetN(plhs[0]) != _cols) {
    sprintf(buf,"Function %s returned a matrix of wrong size",_name);  
    MyIpoptError(buf);
  }
  if ( mxIsSparse(plhs[0]) )
    *_isSparse = true;
  else 
    *_isSparse = false;

  /******************** free memory ***********************/
  mxDestroyArray(plhs[0]);                                                                                                                             
  mxDestroyArray(numOfElements[0]);                                                                                                                    
  mxFree(plhs);                                                                                                                                       
  mxFree(numOfElements);             
  destroyPRHS(analyze_prhs,_method,_nrhs,G_NoOfConstraints,0);
  mxFree(analyze_prhs);
  
  if (*_isSparse == true) {
    returnval = AnalyzeSparseMatrix(_rows, _cols, _n, _nrhs,
				    _name, _method, _reassurance,
				    _m, CalcJacobian, CalcHessian); 
  }
  else if (*_isSparse == false) {   
    STRLEN =  mxGetN( G_FuncNames[4] ) * sizeof(mxChar) + 1;
    mxGetString(G_FuncNames[4],tempString,STRLEN);
    if ( strcmp(_name,tempString) == 0 && CalcHessian == true ) {
      G_hessefixedrow = CreateFintVector(_rows*_cols);
      G_hessefixedcol = CreateFintVector(_rows*_cols);
      CallMatlabFull(_n*_n, _n, _nrhs, 1, testx, dummy, _n, _n,
		     _n, _n, 0, _name, testlambda, G_hessefixedrow,
		     G_hessefixedcol, 0,_method);
    }
    STRLEN =  mxGetN( G_FuncNames[3] ) * sizeof(mxChar) + 1;
    mxGetString(G_FuncNames[3],tempString,STRLEN);
    if ( strcmp(_name,tempString) == 0 && CalcJacobian == true ) {
      G_jacobifixedrow = CreateFintVector(_rows*_cols);
      G_jacobifixedcol = CreateFintVector(_rows*_cols);
      CallMatlabFull(_n*_m,_n,_nrhs,1,testx,dummy,_m,_n,
		     _n,_m,0,_name,testlambda,G_jacobifixedrow,
		     G_jacobifixedcol,0,_method);
    }
    returnval = _rows*_cols;
  }

  /******************** free memory ***************************/
  if (_m > 0 )
    mxFree(testlambda);
  mxFree(testx);
  mxFree(besttestx);
  MyDebug("Leaving AnalyzeMatrix\n",1);
  return returnval;
}


/*******************************************************************
 * int eval_F
 *******************************************************************
 * Compute the value of the objective function
 *
 * n:   number of optimization variables
 * x:   values of optimization variables at which objective function
 *      is to be evaluated
 * f:   value of the objective function
 *******************************************************************/
int eval_F(fint _n, double *_x, double *_f, void* _userData)
{
  fint*   dummy2     = NULL; 
  int     STRLEN     = 0;
  double* dummy      = NULL;

  MyDebug("\nAt start of eval_F\n",1);
  STRLEN = MSIZE(G_FuncNames[0]) * sizeof(mxChar) + 1;
  mxGetString(G_FuncNames[0],tempString,STRLEN); 

  CallMatlabFull(0,_n,2,1,_x,_f,1,1,_n,1,1,
		 tempString,dummy,dummy2,dummy2,1,-1);   

  /* Add current values to tracing vectors */
  if (G_Trace == true)
    G_XVector    = AddToVector(_n,G_XVector,_x);
  if (G_Trace == true)
    G_ObjVector = AddToVector(1,G_ObjVector,_f);
  G_Iterationcount++;
  MyDebug("\nLeaving eval_F\n",1); 
  return 0;
}


/*************************************************************
 * int eval_G
 *************************************************************
 * Compute the gradient of the objective function
 *
 * _n:   number of optimization variables
 * _x:   values of optimization variables at which gradient
 *       is to be evaluated
 * _g:   value of the gradient
 ************************************************************/
int eval_G(fint _n, double* _x, double* _g, void* _userData)
{
  fint*   dummy2 = NULL;
  int     method = 0;
  int     STRLEN = 0;
  double* dummy  = NULL;

  MyDebug("At start of eval_G\n",1);
  
  STRLEN = MSIZE(G_FuncNames[2]) * sizeof(mxChar) + 1;
  mxGetString(G_FuncNames[2],tempString,STRLEN);

  if ( strcmp(tempString,"IpoptADiMat") == 0 )
    CallMatlabFull(0,_n,5,1,_x,_g,_n,1,_n,1,_n,
		   tempString,dummy,dummy2,dummy2,1,0);
  else if ( !strcmp(tempString,"IpoptFD")) {
    method = 10 + (int) *(mxGetPr(G_FD_Params)+7);
    CallMatlabFull(0,_n,8,1,_x,_g,_n,1,_n,1,_n,
		   tempString,dummy,dummy2,dummy2,1,method);
  }
  else 
    CallMatlabFull(0,_n,2,1,_x,_g,_n,1,_n,1,_n,
		   tempString,dummy,dummy2,dummy2,1,-1);
  MyDebug("Leaving eval_G\n",1);
  return 0;
}


/******************************************************************
 * eval_C
 *****************************************************************
 * Compute the values of the constraints
 *
 * _n:   number of optimization variables
 * _x:   values of optimization variables at which constraints
 *       are to be evaluated
 * _m:   number of constraints
 * _c:   values of the constraints
 ******************************************************************/
int eval_C(fint _n, double *_x, fint _m, double *_c, void *_userData)
{
  fint*   dummy2 = NULL;
  int     STRLEN = 0;
  double* dummy  = NULL;

  MyDebug("At start of eval_C\n",1);

  STRLEN = MSIZE(G_FuncNames[1]) * sizeof(mxChar) + 1;
  
  mxGetString(G_FuncNames[1],tempString,STRLEN);
  CallMatlabFull(0,_n,2,1,_x,_c,_m,1,_n,1,_m,
		 tempString,dummy,dummy2,dummy2,1,-1); 
  MyDebug("Leaving eval_c\n",1);
  return 0;
}

/***************************************************************
 * int eval_A
 ***************************************************************
 * Compute the Jacobian of the constraints
 *
 * _task: if 0,  only return number of nonzeros (nz)
 *        otherwise:
 *
 * _n:    number of optimization variables
 * _x:    values of optimization variables at which constraint
 *        Jacobian is to be evaluated
 * _nz:   number of nonzeros in Jacobian
 * _A:    values of the Jacobian entries
 * _Acon: row indices of the Jacobian entries
 * _Avar: column indices of the Jacobian entries
 * G_NoOfConstraints:     global variable = number of constraints   
 ****************************************************************/
int eval_A(fint _task, fint _n, double *_x, fint *_nz,
           double *_A, fint *_Acon, fint *_Avar, void *_userData) 
{
  int     method     = 0;
  int     STRLEN     = 0;
  double* dummy      = NULL;

  MyDebug("At start of eval_a ",1);

  if (_task == 0) {
    *_nz = G_nonzerosjac;
    return 0;
  }
  STRLEN = MSIZE(G_FuncNames[3]) * sizeof(mxChar) + 1; 

  mxGetString(G_FuncNames[3],tempString,STRLEN);
  memcpy(_Avar,G_jacobifixedcol,*_nz*sizeof(fint));
  memcpy(_Acon,G_jacobifixedrow,*_nz*sizeof(fint));

  if ( !strcmp(tempString,"IpoptADiMat") ) {
    CallMatlabSparse(*_nz,_n,5,1,_x,_A,G_NoOfConstraints,
		     _n,_n,1,*_nz,tempString,dummy,
		     _Acon,_Avar,*_nz,1,1);
  }
  else { 
    if (*G_jacobisparse == false) {
      if ( !strcmp(tempString,"IpoptFD") ) {
	method = 20 + *(mxGetPr(G_FD_Params)+7);
	CallMatlabFull(*_nz,_n,8,1,_x,_A,G_NoOfConstraints,
		       _n,_n,1,*_nz,tempString,dummy,
		       _Acon,_Avar,1,method);
      }
      else {
	CallMatlabFull(_n*G_NoOfConstraints,_n,2,1,_x,_A,
		       G_NoOfConstraints,_n,_n,1,_n*G_NoOfConstraints, 
		       tempString,dummy,_Acon,_Avar,1,-1);  
      }
    }
    else if (*G_jacobisparse == true) {
      if ( !strcmp(tempString,"IpoptFD") ) {
	method = 22 + *(mxGetPr(G_FD_Params)+7);
	CallMatlabSparse(*_nz,_n,8,1,_x,_A,G_NoOfConstraints,
			 _n,_n,1,*_nz,tempString,dummy,
			 _Acon,_Avar,*_nz,1,method);
      }
      else {
	CallMatlabSparse(*_nz,_n,2,1,_x,_A,G_NoOfConstraints,
			 _n,_n,1,*_nz,tempString,dummy,
			 _Acon,_Avar,*_nz,1,-1);  
      }
    }
  }
  MyDebug("Leaving eval_a\n",1);
  return 0;
}


/*****************************************************************
 * int eval_H
 ****************************************************************
 * Compute the Hessian of the Lagrangian function
 *
 * _task  : if 0, the matrix gets analyzed     
 * nonzerosfound  
 *        : Number of non zero elements found      
 * _n     : number of optimization variables
 * _x     : values of optimization variables at which constraint
 *          Jacobian is to be evaluated
 * _lambda: values of multipliers
 * _nnzh  : number of nonzeros in Jacobian
 * _hess  : values of the Hessian entries
 * _irnh  : row indices of the Hessian entries
 * _icnh  : column indices of the Hessian entries
 ****************************************************************/
int eval_H(fint _task, fint _n, double *_x, fint _m,
           double *_lambda, fint *_nnzh, double *_hess,
           fint *_irnh, fint *_icnh, void *_userData)
{
  int STRLEN;

  flush();
  MyDebug("At start of eval_h\n",1);

  STRLEN = MSIZE(G_FuncNames[4]) * sizeof(mxChar) + 1;
  mxGetString(G_FuncNames[4],tempString,STRLEN*sizeof(mxChar));

  if ( !strcmp(tempString,"") )
    MyIpoptError("eval_H not provided, but needed with chosen settings");

  if (_task == 0){ 
    *_nnzh = G_nonzeroshess;
    return 0;
  }
  memcpy(_icnh,G_hessefixedcol,*_nnzh*sizeof(fint));
  memcpy(_irnh,G_hessefixedrow,*_nnzh*sizeof(fint));
  
  if (strcmp(tempString,"IpoptADiMat") == 0 ) {
    CallMatlabSparse(*_nnzh,_n,7,1,_x,_hess,_n,_n,_n,1,*_nnzh,
		     tempString,_lambda,_irnh,_icnh,*_nnzh,1,2);
  }
  else  {
    if (*G_hessesparse == false) {
      if (strcmp(tempString,"IpoptFD") == 0 )
	CallMatlabFull(*_nnzh,_n,8,1,_x,_hess,_n,_n,_n,_n,_n*_n,
		       tempString,_lambda,_irnh,_icnh,1,30);
      else
	CallMatlabFull(_n*_n,_n,3,1,_x,_hess,_n,_n,_n,1,_n*_n,
		       tempString,_lambda,_irnh,_icnh,1,-1); 
    }   
    else if (*G_hessesparse == true) {
      if (strcmp(tempString,"IpoptFD") == 0 )
	CallMatlabSparse(*_nnzh,_n,8,1,_x,_hess,_n,_n,_n,1,*_nnzh,
			 tempString,_lambda,_irnh,_icnh,*_nnzh,1,31);
      else
	CallMatlabSparse(*_nnzh,_n,3,1,_x,_hess,_n,_n,_n,1,*_nnzh,
			 tempString,_lambda,_irnh,_icnh,*_nnzh,1,-1); 
      sprintf(buf,"Nonzeros = nnzh = %d",(int) *_nnzh); 
      MyDebug(buf,10);
    }
  }
  MyDebug("Leaving eval_h besser\n",1);
  return 0;
}
