/**********************************************************************
 * $Id$
 * $Source$
 *
 * Part of Scilab Mex interface for IPOPT
 *
 * This code is published under the Common Public License
 * http://www.opensource.org/licenses/cpl.php
 *
 * Copyright (C) 2004; All Rights Reserved
 *
 * Claas Michalik <Michalik<at>LPT<dot>RWTH-Aachen<dot>de>
 * Process Systems Engineering Institute, RWTH Aachen
 * 
 * Steinar Hauan <hauan@cmu.edu>
 * Dept of ChemE, Carnegie Mellon University
 * 
 * Modifications to Scilab Interface made by
 * Edson Cordeiro do Valle <edsoncv@vrtech.com.br>
 * Departamento de Desenvolvimento e Tecnologia, VRTech Tecnologias Industriais LTDA.
 * www.vrtech.com.br
 **********************************************************************/  


#include "IpoptMex.h"


const  int G_NoOfExtraFDParams = 4; /* number of additional parameters
				       needed for the FD algorithm  */

/**** global variables holding general information and parameters **/
fint     G_NoOfConstraints; /* number of constraints               */ 
int      G_debug      = 0;  /* print debug info if TRUE            */ 
int      G_printLevel = 5;  /* determines how much info is printed */ 
int      G_reassurance;     /* determines how intensively matrices 
                               are analysed                        */  
mxArray* G_FD_Params = NULL;/* vector containig parameters for the 
                               finite differences derivative       
                               calculation                         */ 
mxArray* G_Params1   = NULL;/* vector containing the parameters    
                               that are passed to any MATLAB       
                               function                            */ 

/**** global mxArray holding function names for MATLAB callbacks ***/
mxArray** G_FuncNames = NULL;

/******* global variables needed to store tracing information ******/
bool     G_Trace          = false;
int      G_Iterationcount = 0;
double*  G_XVector = NULL;
double*  G_ObjVector = NULL;

/*********** global variables used for sparsity analysis ***********/
bool* G_jacobisparse   = NULL;      /*true if jacobbian is sparse  */
bool* G_hessesparse    = NULL;      /*true if hesseian is sparse   */
fint* G_jacobifixedrow = NULL;      /*row-indices of jacobain      */
fint* G_jacobifixedcol = NULL;      /*column-indices of jacobian   */
fint* G_hessefixedrow  = NULL;      /*row-indices of hessian       */
fint* G_hessefixedcol  = NULL;      /*column-indices of hessain    */
int   G_nonzerosjac    = 0;         /*number of non zero elements in
                                      jacobian                     */   
int   G_nonzeroshess   = 0;         /*num. of non zeros in hessian */     
mxArray* G_FDJacobianPattern = NULL;/*sparsity structure of Jacobian
				      needed for FD-algorithm      */ 
mxArray* G_FDHessianPattern  = NULL;/*sparsity structure of Hessian
				      needed for FD-algorithm      */ 

/********* Global variables holding information on bounds ***********/
fint*   G_ilb    = NULL;             /* inidces of lower bounds     */
fint*   G_iub    = NULL;             /* indices of upper bounds     */
fint    G_nlb    = 0;                /* number of lower bounds      */
fint    G_nub    = 0;                /* number of upper bounds      */
double* G_bnds_u = NULL;             /* values of lower bounds      */
double* G_bnds_l = NULL;             /* values of upper bounds      */

char* buf = NULL;          /* temporary string storage for messages */
char* buf2 = NULL;         /* temporary string storage for messages */
char* tempString = NULL;   /* temporary string storage for messages */


/**********************************************************************
 * mexFunction is MATLAB's hook into our C file                       
 **********************************************************************/

#define MYSELF  "IPOPT"

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
     
{
  /* The varibales :
   * admat    : true iff automatic differentiation is used to
   *            calculate derivatives 
   * fd       : true iff finite differences are used to calculate
   *            derivatives 		
   * ADiMatFunc: Name of the MATLAB function (M-file) to call in order
   *            to calculate derivatives using automatic
   *            differentiation 
   * FiniteDiffFunc: Name of the MATLAB function (M-file) to call in
   *            order to calculate derivatives using finite
   *            differences
   * buf      : used to temproarily store strings
   * buf2     : used to temporarily store strings
   * param    : holds names of the parameters which are to be set  
   * n        : number of optimization variables
   * index    : index of a bound
   * iter     : number of iterations taken
   * retval   : value returned from Ipopt (Errorcode)
   * i,j      : loop counting integers
   * STRLEN   : holds length of strings
   * value    : the value a parameter is to be set to 
   * Obj      : final (optimal ?) value of the objective
   * *v_lb    : multipliers for the lower bounds
   * *v_ub    : multipliers for the upper bounds
   * *lambda  : value of lambda
   * *c       : final maximal value of constraint violation
   * problem  : Ipopt problem which is to be solved 
   * starttime: system time when interface was started
   * endtime  : system time when interface was done
   * difftime : time difference between starttime and endtime
   */

  bool    admat               = false;    
  bool    fd                  = false;  
  char*   ADiMatFunc          = NULL; 
  char*   FiniteDiffFunc      = NULL;
  char*   param               = NULL;
  fint    n                   = 0;
  fint    index               = 0; 
  fint    iter                = 0;
  fint    retval              = 0; 
  int     i                   = 0;
  int     j                   = 0; 
  int     STRLEN              = 0;
  double  value               = 0;
  double  Obj;
  double  retv;
  double* FD_Params           = NULL;
  double* x;
  double* x_start;
  double* v_lb                = NULL;
  double* v_ub                = NULL;
  double* lambda              = NULL;
  double* c                   = NULL; 
  Ipopt   problem             = NULL;
  long double  starttime, endtime;
  
  /* Variables needed to analyze matrices */
  int     nrhsJac             =  2;
  int     nrhsHess            =  3;
  int     jacmethod           = -1;
  int     hessmethod          = -1;
  
  /* allocate memory for variables */
  buf            = (char*    ) mxCalloc(8*BUFLEN,sizeof(char)    );
  buf2           = (char*    ) mxCalloc(4*BUFLEN,sizeof(char)    );
  param          = (char*    ) mxCalloc(BUFLEN,sizeof(char)    );
  ADiMatFunc     = (char*    ) mxCalloc(BUFLEN,sizeof(char)    );
  FiniteDiffFunc = (char*    ) mxCalloc(BUFLEN,sizeof(char)    );
  tempString     = (char*    ) mxCalloc(BUFLEN,sizeof(char)    );
  G_FuncNames    = (mxArray**) mxCalloc(5   ,sizeof(mxArray*));
  G_jacobisparse = (bool*    ) mxCalloc(1   ,sizeof(bool)    );
  G_hessesparse  = (bool*    ) mxCalloc(1   ,sizeof(bool)    );  

  sprintf(ADiMatFunc,"IpoptADiMat");
  sprintf(FiniteDiffFunc,"IpoptFD");

  MyDebug("At start of mex function",1);
  MyInfo ("Execution started ...\n"   ,5);

  /* initialize the random number generator */
  srand(time(NULL) ^ (getpid() + (getpid() << 15)));

  starttime = GetTimeStamp();

  /* set local copies of global variables */ 
  /* set initial/default values for global parameters */
  G_Iterationcount = 0;
  G_printLevel     = 5;
  G_reassurance    = 3;
  /* Set up FD_Params with default values         */
  FD_Params = CreateDoubleVector(10);
  FD_Params[0]   = 0;     /* h1 start             */
  FD_Params[1]   = 0;     /* h2_start             */ 
  FD_Params[2]   = 1E-6;  /* eps1                 */ 
  FD_Params[3]   = 1E-6;  /* eps2                 */ 
  FD_Params[4]   = 10;    /* maxiter1             */
  FD_Params[5]   = 10;    /* maxiter2             */
  FD_Params[6]   = 1;     /* print warnings       */ 
  FD_Params[7]   = 0;     /* use default method   */
  FD_Params[8]   = 0;     /* Jacobisparse = false */
  FD_Params[9]   = 0;	  /* Hessesparse  = false */	  
  /* set default function names for MATLAB callbacks */
  
  char* eval_f_name = (char*) mxCalloc(BUFLEN,sizeof(char));
  sprintf(eval_f_name,"not set");
  char* eval_c_name = (char*) mxCalloc(BUFLEN,sizeof(char)); 
  char* eval_g_name = (char*) mxCalloc(BUFLEN,sizeof(char));
  char* eval_a_name = (char*) mxCalloc(BUFLEN,sizeof(char));
  char* eval_h_name = (char*) mxCalloc(BUFLEN,sizeof(char));       

  if ( nrhs < 12 ){
    sprintf(buf,"%s requires 12+ input arguments (got %d)",MYSELF,nrhs);
    MyIpoptError(buf);
  }
  /* verify correct number if output arguments */ 
  if ( nlhs < 6 || nlhs > 8 ){
    sprintf(buf,"%s requires 6,7 or 8 outputs (got %d)",MYSELF,nlhs);
    MyIpoptError(buf);
  }

  for (j=0; j < 5; j++) {
    if ( mxIsEmpty(prhs[j]) ){
      if ( j == 0 ){
	sprintf(buf,"You must provide an objective function.\n");
	strcat(buf,"The 1st input argument must be a string, i.e. function name.\n");
	MyIpoptError(buf);
      } else if ( j == 2 ) {
	sprintf(buf,"You must provide a gradient function.\n");
	strcat(buf,"The 3rd input argument must be a string, i.e. function name.\n");
	MyIpoptError(buf);
      }
    }
    if ( !mxIsChar(prhs[j]) && !mxIsEmpty(prhs[j]) ){
      sprintf(buf,"Input argument %d is not a string.\n",j+1);
      strcat(buf,"It should correspond to a MATLAB function name");
      MyIpoptError(buf);
    }
  }
  if ( !mxIsNumeric(prhs[5]) || !IS_SCALAR(prhs[5]) ){
    sprintf(buf,"Input 6 must a scalar w/#equalty constraints.\n");
    MyIpoptError(buf);
  }
  for(j=7; j<11; j++)
    if ( !mxIsEmpty(prhs[j]) && !mxIsNumeric(prhs[j]) ){
      sprintf(buf,"Input argument %d must be a vector or empty.\n",j);
      MyIpoptError(buf);
    }
  
  /* number of optimization variables        */
  n                  = (fint)  ( mxGetNumberOfElements(prhs[6]) );
  /* number of equality constraints          */
  G_NoOfConstraints  = (fint) *( mxGetPr(prhs[5]) );
  /* number of lower bounded variables       */
  G_nlb              = (fint)  ( mxGetNumberOfElements(prhs[7]) );
  /* number of upper bounded variables       */
  G_nub              = (fint)  ( mxGetNumberOfElements(prhs[9]) );
	  
  if (G_NoOfConstraints > n) 
    MyIpoptError("The number of equality constratins is not allowed\
                  to be greater than the number of optimization variables!");
  if (G_nlb < 0 || G_nlb > n || G_nub < 0 || G_nub > n)
    MyIpoptError("The number of lower and upper bounded varables has\
                   to fullfill: 0 < X < n \nwith X = number of lower respectively upper\
                   bounds and n = number of optimization variables!"); 
  
  /* Check and set lower bounds */
  if ( mxGetNumberOfElements(prhs[7]) != G_nlb ||
       mxGetNumberOfElements(prhs[8]) != G_nlb ) { 
    sprintf(buf,"Input arguments 8 and 9 have to be vectors of\
                 length nlb with nlb = number of lower bounded variables = %d",
	    (int) G_nlb);
    MyIpoptError(buf);
}
  /** Create vectors holding information on lower bounds if needed **/   
  if ( mxIsEmpty(prhs[7]) == false && mxIsEmpty(prhs[8])== false ) {
    G_ilb    = CreateFintVector(n);    /*indices of lower bounds*/   
    G_bnds_l = CreateDoubleVector(n);  /*values of lower bounds */
  }
  for (i=0; i<G_nlb; i++) {
    index = (fint) *(mxGetPr(prhs[7])+i);
    if ( ! (1 <= index && index <= n) ) 
      MyIpoptError("All indices for lower bounds have to be greater \
                    or equal to zero and less or equal to n with n = number of\
                    optimization variables");
    G_ilb[i] = index;
  }
  for (i=0; i<G_nlb; i++) {
    G_bnds_l[i] = (double) *(mxGetPr(prhs[8])+i);
  }
  /* Check and set upper bounds */
  if ( mxGetNumberOfElements(prhs[9]) != G_nub ||
       mxGetNumberOfElements(prhs[10]) != G_nub ) { 
    sprintf(buf,"Input arguments %d and %d have to be vectors of\
                 length nub with nub = number of upper bounded variables = %d",
	    10,11,(int) G_nub);
    MyIpoptError(buf);
  }
  /* Create vectors for information on upper bounds */       
  if ( mxIsEmpty(prhs[9]) == false && mxIsEmpty(prhs[10]) == false ) {
    G_iub    = CreateFintVector(G_nub);   /*indices of upper bounds */  
    G_bnds_u = CreateDoubleVector(G_nub); /* values of upper bounds */
  }
  for (i=0; i<G_nub; i++) {
    fint index = (fint) *(mxGetPr(prhs[9])+i);
    if( !(1 <= index && index <= n) ) 
      MyIpoptError("All indices of the upper bounds have to be\
                    greater or equal to zero and less or equal to n with n = number of\
                    optimization variables");
    G_iub[i] = index;
  }
  for (i=0; i<G_nub; i++) {
    G_bnds_u[i] = (double) *(mxGetPr(prhs[10])+i);
  }

  /* Copy names of MATLAB functions for callbacks to global variables */
  if ( mxIsEmpty(prhs[0]) != true ) {
    STRLEN =  mxGetN( prhs[0] ) * sizeof(mxChar) + 1;  
    mxGetString(prhs[0],eval_f_name,STRLEN);
  }
  if ( mxIsEmpty(prhs[1]) != true ) {
    if( G_NoOfConstraints == 0 )
      MyIpoptError("Number of constraints is 0 but a constraint function is provided !");
    STRLEN =  mxGetN( prhs[1] ) * sizeof(mxChar) + 1;
    mxGetString(prhs[1],eval_c_name,STRLEN);
  }
  if ( mxIsEmpty(prhs[2]) != true ) {
    STRLEN =  mxGetN( prhs[2] ) * sizeof(mxChar) + 1;
    mxGetString(prhs[2],eval_g_name,STRLEN);
  }
  if ( !strcmp(eval_g_name,"ad") ) {
    memcpy(eval_g_name,ADiMatFunc,BUFLEN*sizeof(char));
    admat = true;
  }
  if ( !strcmp(eval_g_name,"fd") ) {
    memcpy(eval_g_name,FiniteDiffFunc,BUFLEN*sizeof(char));
    fd = true;
  }
  if ( mxIsEmpty(prhs[3]) != true ) {
    STRLEN =  mxGetN( prhs[3] ) * sizeof(mxChar) + 1;
    mxGetString(prhs[3],eval_a_name,STRLEN);
  }
  if ( !strcmp(eval_a_name,"ad") ) {
    memcpy(eval_a_name,ADiMatFunc,BUFLEN*sizeof(char));
    admat = true;
  }
  if ( !strcmp(eval_a_name,"fd") ) {
    memcpy(eval_a_name,FiniteDiffFunc,BUFLEN*sizeof(char));
    fd = true;
    /* G_reassurance = 2 is  default for  this case */
    G_reassurance = 2; 
  }
  if ( mxIsEmpty(prhs[4]) != true ) {
    STRLEN =  mxGetN( prhs[4] ) * sizeof(mxChar) + 1;
    mxGetString(prhs[4],eval_h_name       ,STRLEN);   
  }
  if ( !strcmp(eval_h_name,"ad") ) {
    memcpy(eval_h_name,ADiMatFunc,BUFLEN*sizeof(char));
    admat = true;
  }
  if ( !strcmp(eval_h_name,"fd") ) {
    memcpy(eval_h_name,FiniteDiffFunc,BUFLEN*sizeof(char));
    fd = true;
    /* G_reassurance = 2 = default in this case */
    G_reassurance = 2;
  }
  
  /********** check if all necessary files exist ***********/
  if ( admat == true ) {
    mxArray** test = (mxArray**) mxCalloc(1,sizeof(mxArray*));
    mxArray** name = (mxArray**) mxCalloc(1,sizeof(mxArray*));
    test[0] = mxCreateDoubleScalar(0);
    name[0] = mxCreateString("evalJ");
    mexCallMATLAB(1,test,1,name,"exists");
    mxDestroyArray( test[0] );
    mxDestroyArray( name[0] );
    mxFree(test);
    mxFree(name);
  }
  if ( fd == true ) {
    mxArray** test = (mxArray**) mxCalloc(1,sizeof(mxArray*));
    mxArray** name = (mxArray**) mxCalloc(1,sizeof(mxArray*));
    test[0] = mxCreateDoubleScalar(0);
    name[0] = mxCreateString("IpoptFD");
    mexCallMATLAB(1,test,1,name,"exists");
    if(mxGetScalar(test[0]) != 1)
//    if(mxGetScalar(test[0]) != 2 && mxGetScalar(test[0]) != 3)
      MyIpoptError("You chose at least one derivative to be evaluated\
                    using finite differneces, but file IpoptFD.m is missing !");
    mxDestroyArray( test[0] );
    mxDestroyArray( name[0] );
    mxFree(test);
    mxFree(name);
  }
  
  /* If called with 8 output arguments set G_Trace = true */
  if ( nlhs == 8 ) 
    G_Trace = true;
  
  /* If user provides a starting point -- indicated by the array not
     being empty -- the vector has to be 1 x n or n x 1 -- where n si
     the number of optimization variables */

  if ( mxGetNumberOfElements(prhs[6]) == n )
    if ( (mxGetM(prhs[6]) != 1 && mxGetN(prhs[6]) != 1) ) {
      sprintf(buf,"The provided start point (input argument 7) should either be empty");
      strcat(buf," or a vector of length equal to #optimization variables");
      MyIpoptError(buf);
    }
  
  /* Set the start point */
  x = CreateDoubleVector(n);

  if ( (double) *(mxGetPr(prhs[6])) == 123 ) {
    sprintf(buf,"The provided start-point-vector contains 123 as first values,\n(--> start point will be chosen randomly)");  
    MyInfo(buf,1);
    setRandomValidPoint(x,n); /* set x to a random valid point */
  }
  
  /* set x to user provided start point */
  else if ( mxGetNumberOfElements(prhs[6]) == n ) {
    for (i=0; i<n; i++)
      x[i] = *(mxGetPr(prhs[6])+i);
  }

  x_start = CreateDoubleVector(n);
  for (i=0; i<n; i++)
    x_start[i] = x[i];
  
  MyDebug("After input argument checking\n",1);  
  
  /* Initialize vectors to store trace information with initial
     capacity to store 100 iteratinos. Array will be enlarged if
     necessary */
  if ( G_Trace == true ) {
    G_XVector   = CreateDoubleVector(5*100);
    G_ObjVector = CreateDoubleVector(  100);
  }
  
  /* create local copy of MATLAB variable containing the user provided 
     parameters to be passed to MATLAB functions if provided */ 
  G_Params1 = mxDuplicateArray(prhs[11]);
 
  /* 
   * First create an Ipopt object, which will store certain
   * quantities, such as size of the problem, values of the variables
   * (initial point, and later the solution), etc
   */
  MyDebug("Before Ipopt problem initialization\n",1);     
  problem = Ipopt_Create(n, G_NoOfConstraints, G_nlb, G_ilb, G_bnds_l, 
			 G_nub, G_iub, G_bnds_u, eval_F, eval_C,
			 eval_G, eval_A, eval_H); 
  MyDebug("After ipopt problem initialization\n",1);   

  if ( problem == NULL )
    MyIpoptError("Could not allocate Ipopt problem\n");
  
  /* set user supplied parameters */

  if ( (nrhs%2) != 0 ){ /* even number of rhs */
    sprintf(buf,"You have to provide an even number of input arguments.\n");
    strcat(buf,"Setting a parameter always requires two input arguments;\n");
    strcat(buf,"the first being the parameter name and the second its value.");
    MyIpoptError(buf);
  }

  /* First check printlevel (before printing anything!) */
  for (i = 12; i < nrhs; i += 2) {
    j = i;
    flush();
    if ( !mxIsChar(prhs[j]) )
      MyIpoptError("To set a parameter provide the name as one input\
                   argument(@ an odd position) and than the value (a scalar) you want \
                   to set it to as the next input argument (@ an even position)"); 

    if ( mxGetNumberOfElements(prhs[j+1]) != 1 )
      MyIpoptError("To set a parameter provide the name as one input\
                    argument(@ an odd position) and than the value (a scalar) you want \
                    to set it to as the next input argument (@ an even position)");
    
    STRLEN = mxGetN(prhs[j]) * sizeof(mxChar) + 1;
    mxGetString(prhs[j],param,STRLEN);
    value = mxGetScalar(prhs[j+1]);    /* the user provided value */
    
    /* Taking care of the interface parameters */
    if ( !strcmp(param,"debug") ) 
      G_debug        = (int)    value;
    else if ( !strcmp(param,"print") )
      G_printLevel   = (int)    value;
  }

  /* print information summary to the screen */

  MyInfo("\n******************************************************************************\n",5);

  MyInfo("Problem Summary:\n",5);

  sprintf(buf,"Name of Objective function:                   ");
  strcat(buf, eval_f_name);
  MyInfo(buf,5  );
  mexPrintf("\n");

  sprintf(buf,"Name of Constraint function:                  ");
  strcat(buf, eval_c_name);
  MyInfo(buf,5  );
  mexPrintf("\n");

  sprintf(buf,"Name/Method of Objective Gradient function:   ");
  if ( !strcmp(eval_g_name, "IpoptFD" ) )
    strcat(buf, "Finite Differences"       );
  else if ( !strcmp(eval_g_name, "IpoptADiMat" ) )
    strcat(buf, "Automatic Differentiation ADiMat");
  else 
    strcat(buf, eval_g_name);
  MyInfo(buf,5  );
  mexPrintf("\n");

  sprintf(buf,"Name/Method of Constraints Gradient function: ");
  if ( !strcmp(eval_a_name, "IpoptFD") )
       strcat(buf, "Finite Differences");
  else if ( !strcmp(eval_a_name, "IpoptADiMat" ) )
    strcat(buf, "Automatic Differentiation ADiMat");
  else 
    strcat(buf, eval_a_name);
  MyInfo(buf,5  );
  mexPrintf("\n");

  sprintf(buf,"Name/Method of Hessian function:              ");
  if ( !strcmp(eval_h_name, "IpoptFD"    ) )
    strcat(buf, "Finite Differences");
  else if ( !strcmp(eval_h_name, "IpoptADiMat" ) )
    strcat(buf, "Automatic Differentiation ADiMat");
  else if ( !strcmp(eval_h_name, "") )
    strcat(buf, "Empty - not provided");
  else 
    strcat(buf, eval_h_name);

  MyInfo(buf,5  );
  mexPrintf("\n");

  /* Loop through all additional parameters */
  
  /* Now loop through all parameters and print info to the screen */
  for (i = 12; i < nrhs; i += 2) {
    j = i;
    flush();
    if ( !mxIsChar(prhs[j]) )
      MyIpoptError("To set a parameter provide the name as one input\
                    argument(@ an odd position) and than the value (a scalar) you want \
                    to set it to as the next input argument (@ an even position)"); 

    if ( mxGetNumberOfElements(prhs[j+1]) != 1 )
      MyIpoptError("To set a parameter provide the name as one input\
                    argument(@ an odd position) and than the value (a scalar) you want\
                    to set it to as the next input argument (@ an even position)");

    STRLEN = mxGetN(prhs[j]) * sizeof(mxChar) + 1;
    mxGetString(prhs[j],param,STRLEN);
    value = mxGetScalar(prhs[j+1]);    /* the user provided value */
    
    sprintf(buf, "%-30s",param         );
    strcat (buf, " set to :       "    );
    sprintf(buf2, "%f\n",(double) value);
    strcat (buf,buf2                   );
    MyInfo (buf,5);

    /* Taking care of the interface parameters */
    if ( !strcmp(param,"debug") )
      G_debug        = (int)    value;
    else if ( !strcmp(param,"print") )
      G_printLevel   = (int)    value;
    else if ( !strcmp(param,"findnonzeros") )
	  G_reassurance  = (int)    value;
    else if ( !strcmp(param,"fdh1") )
      FD_Params[0] = (double)   value;
    else if ( !strcmp(param,"fdh2") )
      FD_Params[1] = (double)   value;
    else if ( !strcmp(param,"fdeps1") )
      FD_Params[2] = (double)   value;
    else if ( !strcmp(param,"fdeps2") )
      FD_Params[3] = (double)   value;
    else if ( !strcmp(param,"fdmaxiter1") )
      FD_Params[4] = (double)   value;
    else if ( !strcmp(param,"fdmaxiter2") )
      FD_Params[5] = (double)   value;
    else if ( !strcmp(param,"fdwarnings") )
      FD_Params[6] = (double)   value; 
    else if ( !strcmp(param,"fdmethod") )
      FD_Params[7] = (double)   value > 1 ? 1 : value;
    else if ( !strcmp(param,"fdjacobisparse") )
      FD_Params[8] = (double)   value;
    else if ( !strcmp(param,"fdhessesparse" ) )
      FD_Params[9] = (double)   value;	  	  
    /* Taking care of Ipopt parameters */
    else
      Ipopt_AddParam(problem, param, value);
  }

  /* copy local variables into global mxArrays */
  G_FD_Params = mxCreateDoubleMatrix(1,10,mxREAL);	
  memcpy(mxGetPr(G_FD_Params),FD_Params,10*sizeof(double));

  G_FuncNames[0] = mxCreateString(eval_f_name);
  G_FuncNames[1] = mxCreateString(eval_c_name);
  G_FuncNames[2] = mxCreateString(eval_g_name);
  G_FuncNames[3] = mxCreateString(eval_a_name);
  G_FuncNames[4] = mxCreateString(eval_h_name);
  MyInfo("******************************************************************************\n\n",5);

  /* Set parameters that users are now allowed to change */
  Ipopt_AddParam(problem, "IHESSVECT", 2);
  
  /* set up gradient method  depending values */
  if ( !strcmp(eval_a_name,"IpoptADiMat") ) {
    nrhsJac   = 5;
    jacmethod = 1;
  }
  if ( !strcmp(eval_a_name,"IpoptFD"   ) ) {
    nrhsJac   = 8;
    jacmethod = 21 + 2*FD_Params[8];
  }
  if ( !strcmp(eval_h_name,"IpoptADiMat") ) {
    nrhsHess   = 7;
    hessmethod = 2;
  }
  if ( !strcmp(eval_h_name,"IpoptFD") ) {
    nrhsHess   = 8;
    hessmethod = 30 + FD_Params[9];
  }
  if ( G_NoOfConstraints != 0 ) {
    if ( !strcmp(eval_a_name,"") || !strcmp(eval_c_name,"") ) {
      sprintf(buf,"The problem is subject to %d equality\
                   constraints, but at least one of the functions evaluating the\
                   constraints or the Jacobian of the constraints is empty",
	      (int) G_NoOfConstraints);
      MyIpoptError(buf);
    }
  }
  if ( strcmp(eval_a_name,"") ) { /* if problem is constrained */
    G_nonzerosjac = AnalyzeMatrix(n,G_NoOfConstraints,G_NoOfConstraints,
				  n,nrhsJac,G_reassurance,eval_a_name,
				  G_jacobisparse,jacmethod);
  }   
  if ( strcmp(eval_h_name,"") ) { /* if hessian is provided */
    G_nonzeroshess = AnalyzeMatrix(n,G_NoOfConstraints,n,n,nrhsHess,
				   G_reassurance,eval_h_name,
				   G_hessesparse,hessmethod);
  }

  /* allocate memory for the result from IPOPT */
  if ( G_nlb > 0 )
    v_lb =  CreateDoubleVector(G_nlb);
  if ( G_nub > 0 )
    v_ub =  CreateDoubleVector(G_nub); 
  if ( G_NoOfConstraints>0 ) {
    c      = CreateDoubleVector(G_NoOfConstraints);  
    lambda = CreateDoubleVector(G_NoOfConstraints);
  }
  
  /* call the IPOPT solver subroutines */
  MyDebug("Before IPOPT solve\n",1);  
  flush();

  retval = Ipopt_Solve(problem, (void*)NULL, x, &Obj, v_lb, v_ub,
		       lambda, c, &iter);
  flush();
  MyDebug("After IPOPT solve / before result output\n",1);  

  /* copy outputs to matlab mx structures */
  if ( G_Trace == true ) {
    sprintf(buf,"\n Iteration steps traced : %d",G_Iterationcount);
    MyInfo(buf,10  );
    /* Value history of the Objective */
    plhs[7] = mxCreateDoubleMatrix( 1,G_Iterationcount,mxREAL );
    /* Values history of x            */
    plhs[6] = mxCreateDoubleMatrix( n,G_Iterationcount,mxREAL );
    memcpy(mxGetPr(plhs[7]),(G_ObjVector+1),
           1*(G_Iterationcount)*sizeof(double));
    memcpy(mxGetPr(plhs[6]),(G_XVector+1),
           n*(G_Iterationcount)*sizeof(double));
  }
  if ( nlhs == 7 ) {
    plhs[6] = mxCreateDoubleMatrix(n,1,mxREAL);
    memcpy(mxGetPr(plhs[6]),x_start,n*sizeof(double));
  }
  /* final value of x                            */
  plhs[0] = mxCreateDoubleMatrix(n                , 1, mxREAL);
  /* final value of Objective                    */
  plhs[1] = mxCreateDoubleMatrix(1                , 1, mxREAL);
  /* multipliers for the lower bounds            */
  plhs[2] = mxCreateDoubleMatrix(G_nlb            , 1, mxREAL);
  /* multipliers for the upper boundds           */
  plhs[3] = mxCreateDoubleMatrix(G_nub            , 1, mxREAL);
  /* multipliers for the equality constraints    */
  plhs[4] = mxCreateDoubleMatrix(G_NoOfConstraints, 1, mxREAL);
  /* Ipopt info flag                             */
  plhs[5] = mxCreateDoubleMatrix(1                , 1, mxREAL);
  memcpy( mxGetPr(plhs[0]),x    , n     * sizeof(double)     );
  memcpy( mxGetPr(plhs[1]),&Obj , 1     * sizeof(double)     );
  memcpy( mxGetPr(plhs[2]),v_lb , G_nlb * sizeof(double)     );
  memcpy( mxGetPr(plhs[3]),v_ub , G_nub * sizeof(double)     );
  memcpy( mxGetPr(plhs[4]),
	  lambda,G_NoOfConstraints*sizeof(double)                );
  retv = (double) retval;
  memcpy( mxGetPr(plhs[5]),&retv, 1    * sizeof(double)      );
  
  /* print some output to the screen */
  sprintf(buf, "The final value of the objective function:%15.8e\n",Obj);    
  sprintf(buf2,"\nThe optimal values of X are:\n\n");
  strcat(buf, buf2);
  MyInfo(buf,10);
  for (i=0; i < n; i++) {
    sprintf(buf, "%e\n", x[i]);
    MyInfo(buf,10);
  }
  sprintf(buf,"\nThe multipliers for the lower bounds are:\n\n"); 
  MyInfo(buf, 10);
  for (i=0; i < G_nlb; i++) {
    sprintf(buf,"V_L(%5d) = %15.8e\n",(int)G_ilb[i],(double)v_lb[i]);   
    MyInfo(buf, 10);
  }
  sprintf(buf,"\nThe multipliers for the upper bounds are:\n\n");  
  MyInfo(buf, 10);
  for (i=0; i < G_nub; i++) {
    sprintf(buf,"V_U(%5d) = %15.8e\n",(int)G_iub[i],(double)v_ub[i]); 
    MyInfo(buf,10);
  }
  sprintf(buf,"\nThe multipliers for the equality constraints are:\n\n");
  MyInfo(buf,10);
  for (i=0; i < G_NoOfConstraints; i++) {
    sprintf(buf, "LAM(%5d) = %15.8e\n", i+1, lambda[i]             );  
    MyInfo(buf,10);
  }
  if ( retval != 0 ){
    sprintf(buf,"\nAn error has occurred after %d iterations\n", (int) iter);
    sprintf(buf2, "The error code is %d\n\n",(int) retval);
    strcat(buf ,buf2);
    MyInfo(buf, 1);
  }
  
  /* clean up memory and return to the MATLAB calling environment */

   MyDebug("Before cleaning up \n",1);  
   /*mxFree(buf);
   mxFree(buf2);
   mxFree(tempString);
   mxDestroyArray(G_FuncNames[0]);
   mxDestroyArray(G_FuncNames[1]);
   mxDestroyArray(G_FuncNames[2]);
   mxDestroyArray(G_FuncNames[3]);
   mxDestroyArray(G_FuncNames[4]);
   mxFree(G_FuncNames);
   mxFree(G_hessesparse);
   mxFree(G_jacobisparse);
   if ( G_NoOfConstraints > 0 ) {
     mxFree(lambda);
     mxFree(c);
   }
   if ( G_Trace == true ) {
     mxFree(G_XVector);
     mxFree(G_ObjVector);
   }
   if ( strcmp(eval_a_name,"") != 0 ) {
     mxFree(G_jacobifixedrow   );
     mxFree(G_jacobifixedcol   );
   }
   if ( strcmp(eval_h_name,"") != 0 ) {
     mxFree(G_hessefixedrow  );
     mxFree(G_hessefixedcol  );
   }
   if ( G_nub > 0 )
     mxFree(v_ub );
   if ( G_nlb > 0 )
     mxFree(v_lb );

   Ipopt_Destroy(problem);*/

   /* report total timing data */
   endtime = GetTimeStamp();
   sprintf(buf,"\n\nTotal runtime in seconds: %.3f \n\n",
	   (double) (endtime-starttime));
   MyInfo(buf,10);
}
